@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -19,6 +19,7 @@
 
 #include "config.h"
 #include <errno.h>
+#include <getopt.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
@@ -31,7 +32,9 @@
 #endif
 
 #include "boxes.h"
+#include "bxstring.h"
 #include "discovery.h"
+#include "logging.h"
 #include "query.h"
 #include "tools.h"
 #include "cmdline.h"
@@ -56,6 +59,9 @@ extern int optind;     /* for getopt() */
     #define EOL_DEFAULT "\n"
 #endif
 
+#define EXTRA_UNDOC "(undoc)"
+#define EXTRA_DEBUG "debug"
+
 
 
 /**
@@ -76,31 +82,39 @@ static void usage_short(FILE *st)
  */
 void usage_long(FILE *st)
 {
-    char *config_file = discover_config_file(0);
+    bxstr_t *config_file = discover_config_file(0);
 
     fprintf(st, "%s - draws any kind of box around your text (or removes it)\n", PROJECT);
     fprintf(st, "        Website: https://boxes.thomasjensen.com/\n");
     fprintf(st, "Usage:  %s [options] [infile [outfile]]\n", PROJECT);
-    fprintf(st, "        -a fmt   alignment/positioning of text inside box [default: hlvt]\n");
-    fprintf(st, "        -c str   use single shape box design where str is the W shape\n");
-    fprintf(st, "        -d name  box design [default: first one in file]\n");
-    fprintf(st, "        -e eol   Override line break type (experimental) [default: %s]\n",
-                                  strcmp(EOL_DEFAULT, "\r\n") == 0 ? "CRLF" : "LF");
-    fprintf(st, "        -f file  configuration file [default: %s]\n", config_file != NULL ? config_file : "none");
-    fprintf(st, "        -h       print usage information\n");
-    fprintf(st, "        -i mode  indentation mode [default: box]\n");
-    fprintf(st, "        -k bool  leading/trailing blank line retention on removal\n");
-    fprintf(st, "        -l       list available box designs w/ samples\n");
-    fprintf(st, "        -m       mend box, i.e. remove it and redraw it afterwards\n");
-    fprintf(st, "        -n enc   Character encoding of input and output [default: %s]\n", locale_charset());
-    fprintf(st, "        -p fmt   padding [default: none]\n");
-    fprintf(st, "        -q qry   query the list of designs by tag\n"); /* with "(undoc)" as query, trigger undocumented webui stuff instead */
-    fprintf(st, "        -r       remove box\n");
-    fprintf(st, "        -s wxh   box size (width w and/or height h)\n");
-    fprintf(st, "        -t str   tab stop distance and expansion [default: %de]\n", DEF_TABSTOP);
-    fprintf(st, "        -v       print version information\n");
+    fprintf(st, "  -a, --align <fmt>     Alignment/positioning of text inside box [default: hlvt]\n");
+    fprintf(st, "  -c, --create <str>    Use single shape box design where str is the W shape\n");
+    fprintf(st, "      --color           Force output of ANSI sequences if present\n");
+    fprintf(st, "      --no-color        Force monochrome output (no ANSI sequences)\n");
+    fprintf(st, "  -d, --design <name>   Box design [default: first one in file]\n");
+    fprintf(st, "  -e, --eol <eol>       Override line break type (experimental) [default: %s]\n",
+                                         strcmp(EOL_DEFAULT, "\r\n") == 0 ? "CRLF" : "LF");
+    fprintf(st, "  -f, --config <file>   Configuration file [default: %s]\n",
+                                         config_file != NULL ? bxs_to_output(config_file) : "none");
+    fprintf(st, "  -h, --help            Print usage information\n");
+    fprintf(st, "  -i, --indent <mode>   Indentation mode [default: box]\n");
+    fprintf(st, "  -k <bool>             Leading/trailing blank line retention on removal\n");
+    fprintf(st, "      --kill-blank      Kill leading/trailing blank lines on removal (like -k true)\n");
+    fprintf(st, "      --no-kill-blank   Retain leading/trailing blank lines on removal (like -k false)\n");
+    fprintf(st, "  -l, --list            List available box designs w/ samples\n");
+    fprintf(st, "  -m, --mend            Mend (repair) box\n");
+    fprintf(st, "  -n, --encoding <enc>  Character encoding of input and output [default: %s]\n", locale_charset());
+    fprintf(st, "  -p, --padding <fmt>   Padding [default: none]\n");
+    fprintf(st, "  -q, --tag-query <qry> Query the list of designs by tag\n");
+    fprintf(st, "  -r, --remove          Remove box\n");
+    fprintf(st, "  -s, --size <wxh>      Box size (width w and/or height h)\n");
+    fprintf(st, "  -t, --tabs <str>      Tab stop distance and expansion [default: %de]\n", DEF_TABSTOP);
+    fprintf(st, "  -v, --version         Print version information\n");
+    /* fprintf(st, "  -x, --extra <arg>     If <arg> starts with "debug:", activate debug logging for specified log
+                areas which follow in a comma-separated list [default area: MAIN]. If <arg> is "(undoc)", trigger
+                undocumented behavior of design detail lister.");  // undocumented */
 
-    BFREE(config_file);
+    bxs_free(config_file);
 }
 
 
@@ -109,11 +123,13 @@ static opt_t *create_new_opt()
 {
     opt_t *result = (opt_t *) calloc(1, sizeof(opt_t));
     if (result != NULL) {
-        /* all valued initialized with 0 or NULL */
+        /* all values initialized with 0 or NULL */
+        result->color = color_from_terminal;
         result->tabstop = DEF_TABSTOP;
         result->eol = "\n";      /* we must default to "\n" instead of EOL_DEFAULT as long as stdout is in text mode */
         result->tabexp = 'e';
         result->killblank = -1;
+        result->debug = (int *) calloc(NUM_LOG_AREAS, sizeof(int));
         for (int i = 0; i < NUM_SIDES; ++i) {
             result->padding[i] = -1;
         }
@@ -451,6 +467,54 @@ static int size_of_box(opt_t *result, char *optarg)
 
 
 
+static int debug_areas(opt_t *result, char *optarg)
+{
+    char *dup = NULL;
+    if (optarg != NULL) {
+        dup = strdup(optarg);   /* required because strtok() modifies its input */
+    }
+    else {
+        dup = strdup(log_area_names[MAIN]);
+    }
+
+    for (char *a = strtok(dup, ","); a != NULL; a = strtok(NULL, ","))
+    {
+        char *trimmed = trimdup(a, a + strlen(a) - 1);
+        if (strlen(trimmed) == 0) {
+            BFREE(trimmed);
+            continue;
+        }
+        int valid = 0;
+        for (size_t i = ALL; i < NUM_LOG_AREAS + 2; i++) {
+            if (strcasecmp(trimmed, log_area_names[i]) == 0) {
+                valid = 1;
+                if (i == ALL) {
+                    for (size_t i = 0; i < NUM_LOG_AREAS; i++) {
+                        result->debug[i] = 1;
+                    }
+                }
+                else {
+                    result->debug[i - 2] = 1;
+                }
+                break;
+            }
+        }
+        if (!valid) {
+            bx_fprintf(stderr, "%s: invalid debug area -- %s. Valid values: ", PROJECT, trimmed);
+            for (size_t i = 1; i < NUM_LOG_AREAS + 2; i++) {
+                bx_fprintf(stderr, "%s%s", log_area_names[i], i < (NUM_LOG_AREAS + 2 - 1) ? ", " : "\n");
+            }
+            BFREE(trimmed);
+            return 1;
+        }
+        BFREE(trimmed);
+    }
+    BFREE(dup);
+    return 0;
+}
+
+
+
 /**
  * Tab handling. Format is `n[eku]`.
  * @param result the options struct we are building
@@ -498,6 +562,48 @@ static int tab_handling(opt_t *result, char *optarg)
 
 
 
+/**
+ * Handle undocumented options (-x).
+ * @param result the options struct we are building
+ * @param optarg the argument to `-x` on the command line
+ * @returns 0 on success, anything else on error
+ */
+static int undocumented_options(opt_t *result, char *optarg)
+{
+    char *s = optarg;
+
+    if (strncmp(s, EXTRA_UNDOC, strlen(EXTRA_UNDOC)) == 0) {
+        result->qundoc = 1;
+        s += strlen(EXTRA_UNDOC);
+    }
+
+    if (strncasecmp(s, EXTRA_DEBUG, strlen(EXTRA_DEBUG)) == 0) {
+        s += strlen(EXTRA_DEBUG);
+        if (*s == ':') {
+            if (debug_areas(result, s + 1) != 0) {
+                return 1;
+            }
+        }
+        else if (*s == '\0') {
+            /* default to MAIN */
+            debug_areas(result, NULL);
+        }
+        else {
+            bx_fprintf(stderr, "%s: invalid option -x %s\n", PROJECT, optarg);
+            return 2;
+        }
+        activate_debug_logging(result->debug);
+    }
+
+    if (!result->qundoc && !is_debug_activated()) {
+        bx_fprintf(stderr, "%s: invalid option -x %s\n", PROJECT, optarg);
+        return 3;
+    }
+    return 0;
+}
+
+
+
 /**
  * Set *stdout* to binary mode, so that we can control the line terminator. This function only ever does anything on
  * Windows, because on Linux, we already do have control over line terminators.
@@ -576,35 +682,48 @@ static int input_output_files(opt_t *result, char *argv[], int optind)
 
 static void print_debug_info(opt_t *result)
 {
-    if (result != NULL) {
-        #if defined(DEBUG)
-            fprintf (stderr, "Command line option settings (excerpt):\n");
-            fprintf (stderr, "- Alignment (-a): horiz %c, vert %c\n",
-                    result->halign ? result->halign : '?', result->valign ? result->valign : '?');
-            fprintf (stderr, "- Line justification (-a): \'%c\'\n", result->justify ? result->justify : '?');
-            fprintf (stderr, "- Design Definition W shape (-c): %s\n", result->cld ? result->cld : "n/a");
-            fprintf (stderr, "- Line terminator used (-e): %s\n",
-                strcmp(result->eol, "\r\n") == 0 ? "CRLF" : (strcmp(result->eol, "\r") == 0 ? "CR" : "LF"));
-            fprintf (stderr, "- Explicit config file (-f): %s\n", result->f ? result->f : "no");
-            fprintf (stderr, "- Indentmode (-i): \'%c\'\n", result->indentmode ? result->indentmode : '?');
-            fprintf (stderr, "- Kill blank lines (-k): %d\n", result->killblank);
-            fprintf (stderr, "- Mend box (-m): %d\n", result->mend);
-            fprintf (stderr, "- Padding (-p): l:%d t:%d r:%d b:%d\n",
-                    result->padding[BLEF], result->padding[BTOP], result->padding[BRIG], result->padding[BBOT]);
-            fprintf (stderr, "- Tag Query / Special handling for Web UI (-q): ");
-            if (result->query != NULL) {
-                for (size_t qidx = 0; result->query[qidx] != NULL; ++qidx) {
-                    fprintf(stderr, "%s%s", qidx > 0 ? ", " : "", result->query[qidx]);
-                }
-            } else {
-                fprintf (stderr, "(none)");
+    if (result != NULL && is_debug_logging(MAIN)) {
+        log_debug(__FILE__, MAIN, "Command line option settings (excerpt):\n");
+        log_debug(__FILE__, MAIN, "  - Alignment (-a): horiz %c, vert %c\n",
+                result->halign ? result->halign : '?', result->valign ? result->valign : '?');
+        log_debug(__FILE__, MAIN, "  - Line justification (-a): \'%c\'\n", result->justify ? result->justify : '?');
+        log_debug(__FILE__, MAIN, "  - Design Definition W shape (-c): %s\n", result->cld ? result->cld : "n/a");
+        log_debug(__FILE__, MAIN, "  - Color mode: %d\n", result->color);
+
+        log_debug(__FILE__, MAIN, "  - Debug areas (-x debug:...): ");
+        int dbgfirst = 1;
+        for (size_t i = 0; i < NUM_LOG_AREAS; i++) {
+            if (result->debug[i]) {
+                log_debug_cont(MAIN, "%s%s", dbgfirst ? "" : ", ", log_area_names[i + 2]);
+                dbgfirst = 0;
             }
-            fprintf (stderr, "\n");
-            fprintf (stderr, "- Remove box (-r): %d\n", result->r);
-            fprintf (stderr, "- Requested box size (-s): %ldx%ld\n", result->reqwidth, result->reqheight);
-            fprintf (stderr, "- Tabstop distance (-t): %d\n", result->tabstop);
-            fprintf (stderr, "- Tab handling (-t): \'%c\'\n", result->tabexp);
-        #endif
+        }
+        log_debug_cont(MAIN, "%s\n", dbgfirst ? "(off)" : "");
+
+        log_debug(__FILE__, MAIN, "  - Line terminator used (-e): %s\n",
+            strcmp(result->eol, "\r\n") == 0 ? "CRLF" : (strcmp(result->eol, "\r") == 0 ? "CR" : "LF"));
+        log_debug(__FILE__, MAIN, "  - Explicit config file (-f): %s\n", result->f ? result->f : "no");
+        log_debug(__FILE__, MAIN, "  - Indentmode (-i): \'%c\'\n", result->indentmode ? result->indentmode : '?');
+        log_debug(__FILE__, MAIN, "  - Kill blank lines (-k): %d\n", result->killblank);
+        log_debug(__FILE__, MAIN, "  - Mend box (-m): %d\n", result->mend);
+        log_debug(__FILE__, MAIN, "  - Padding (-p): l:%d t:%d r:%d b:%d\n",
+                result->padding[BLEF], result->padding[BTOP], result->padding[BRIG], result->padding[BBOT]);
+
+        log_debug(__FILE__, MAIN, "  - Tag Query (-q): ");
+        if (result->query != NULL) {
+            for (size_t qidx = 0; result->query[qidx] != NULL; ++qidx) {
+                log_debug_cont(MAIN, "%s%s", qidx > 0 ? ", " : "", result->query[qidx]);
+            }
+        } else {
+            log_debug_cont(MAIN, "(none)");
+        }
+        log_debug_cont(MAIN, "\n");
+
+        log_debug(__FILE__, MAIN, "  - qundoc (-x): %d\n", result->qundoc);
+        log_debug(__FILE__, MAIN, "  - Remove box (-r): %d\n", result->r);
+        log_debug(__FILE__, MAIN, "  - Requested box size (-s): %ldx%ld\n", result->reqwidth, result->reqheight);
+        log_debug(__FILE__, MAIN, "  - Tabstop distance (-t): %d\n", result->tabstop);
+        log_debug(__FILE__, MAIN, "  - Tab handling (-t): \'%c\'\n", result->tabexp);
     }
 }
 
@@ -612,50 +731,88 @@ static void print_debug_info(opt_t *result)
 
 opt_t *process_commandline(int argc, char *argv[])
 {
-    #ifdef DEBUG
-        fprintf(stderr, "argc = %d\n", argc);
-        fprintf(stderr, "argv = [");
+    if (is_debug_logging(MAIN)) {
+        log_debug(__FILE__, MAIN, "argc = %d\n", argc);
+        log_debug(__FILE__, MAIN, "argv = [");
         for(int i=0; i<=argc; i++) {
-            fprintf(stderr, "%s%s", argv[i], i < argc ? ", " : "");
+            log_debug_cont(MAIN, "%s%s", argv[i], i < argc ? ", " : "]\n");
         }
-        fprintf(stderr, "]\n");
-    #endif
+    }
 
     opt_t *result = create_new_opt();
-    optind = 1;   /* ensure that getopt() will process all arguments, even in unit test situations */
 
-    /* Intercept '--help' and '-?' cases first, as they are not supported by getopt() */
-    if (argc >= 2 && argv[1] != NULL && (strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") == 0)) {
+    /* Intercept '-?' case first, as it is not supported by getopt_long() */
+    if (argc >= 2 && argv[1] != NULL && strcmp(argv[1], "-?") == 0) {
         result->help = 1;
         return result;
     }
 
+    optind = 1;   /* ensure that getopt() will process all arguments, even in unit test situations */
+    int option_index = 0;
+    const struct option long_options[] = {
+        { "align",         required_argument, NULL, 'a' },
+        { "create",        required_argument, NULL, 'c' },
+        { "color",         no_argument,       NULL, OPT_COLOR },
+        { "no-color",      no_argument,       NULL, OPT_NO_COLOR },
+        { "design",        required_argument, NULL, 'd' },
+        { "eol",           required_argument, NULL, 'e' },
+        { "config",        required_argument, NULL, 'f' },
+        { "help",          no_argument,       NULL, 'h' },
+        { "indent",        required_argument, NULL, 'i' },
+        { "kill-blank",    no_argument,       NULL, OPT_KILLBLANK },
+        { "no-kill-blank", no_argument,       NULL, OPT_NO_KILLBLANK },
+        { "list",          no_argument,       NULL, 'l' },
+        { "mend",          no_argument,       NULL, 'm' },
+        { "encoding",      required_argument, NULL, 'n' },
+        { "padding",       required_argument, NULL, 'p' },
+        { "tag-query",     required_argument, NULL, 'q' },
+        { "remove",        no_argument,       NULL, 'r' },
+        { "size",          required_argument, NULL, 's' },
+        { "tabs",          required_argument, NULL, 't' },
+        { "version",       no_argument,       NULL, 'v' },
+        { "extra",         required_argument, NULL, 'x' },
+        { NULL,            0,                 NULL,  0  }
+    };
+    const char *short_options = "a:c:d:e:f:hi:k:lmn:p:q:rs:t:vx:";
+
     int oc;   /* option character */
     do {
-        oc = getopt(argc, argv, "a:c:d:e:f:hi:k:lmn:p:q:rs:t:v");
+        oc = getopt_long(argc, argv, short_options, long_options, &option_index);
 
         switch (oc) {
 
             case 'a':
                 if (alignment(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 'c':
                 if (command_line_design(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
+            
+            case OPT_COLOR:
+                result->color = force_ansi_color;
+                break;
+
+            case OPT_NO_COLOR:
+                result->color = force_monochrome;
+                break;
 
             case 'd':
                 if (design_choice(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 'e':
                 if (eol_override(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
@@ -663,22 +820,36 @@ opt_t *process_commandline(int argc, char *argv[])
             case 'f':
                 result->f = strdup(optarg);   /* input file */
                 break;
-
+            
             case 'h':
                 result->help = 1;
                 return result;
 
             case 'i':
                 if (indentation_mode(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 'k':
                 if (killblank(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
+            
+            case OPT_KILLBLANK:
+                if (result->killblank == -1) {
+                    result->killblank = 1;
+                }
+                break;
+
+            case OPT_NO_KILLBLANK:
+                if (result->killblank == -1) {
+                    result->killblank = 0;
+                }
+                break;
 
             case 'l':
                 result->l = 1;   /* list available box styles */
@@ -694,18 +865,21 @@ opt_t *process_commandline(int argc, char *argv[])
                 result->encoding = strdup(optarg);   /* character encoding */
                 if (result->encoding == NULL) {
                     perror(PROJECT);
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 'p':
                 if (padding(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 'q':
                 if (query(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
@@ -716,12 +890,14 @@ opt_t *process_commandline(int argc, char *argv[])
 
             case 's':
                 if (size_of_box(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
 
             case 't':
                 if (tab_handling(result, optarg) != 0) {
+                    BFREE(result);
                     return NULL;
                 }
                 break;
@@ -730,10 +906,18 @@ opt_t *process_commandline(int argc, char *argv[])
                 result->version_requested = 1;   /* print version number */
                 return result;
 
+            case 'x':
+                if (undocumented_options(result, optarg) != 0) {
+                    BFREE(result);
+                    return NULL;
+                }
+                break;
+
             case ':':
             case '?':
                 /* Missing argument or illegal option - do nothing else */
                 usage_short(stderr);
+                BFREE(result);
                 return NULL;
 
             case EOF:
@@ -742,11 +926,13 @@ opt_t *process_commandline(int argc, char *argv[])
 
             default:
                 bx_fprintf(stderr, "%s: internal error\n", PROJECT);
+                BFREE(result);
                 return NULL;
         }
     } while (oc != EOF);
 
     if (input_output_files(result, argv, optind) != 0) {
+        BFREE(result);
         return NULL;
     }
 
@@ -755,4 +941,4 @@ opt_t *process_commandline(int argc, char *argv[])
 }
 
 
-/*EOF*/                                                  /* vim: set sw=4: */
+/* vim: set sw=4: */
