@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -20,10 +20,12 @@
 #ifndef BOXES_DISCOVERY_H
 #define BOXES_DISCOVERY_H
 
+#include "bxstring.h"
 
-char *discover_config_file(const int global_only);
+
+bxstr_t *discover_config_file(const int global_only);
 
 
 #endif /* BOXES_DISCOVERY_H */
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
