@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,21 +18,27 @@
  */
 
 #include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
+#include <unictype.h>
+#include <unistr.h>
 
 #include "discovery.h"
-#include "tools.h"
-#include "regulex.h"
-#include "unicode.h"
-#include "query.h"
+#include "logging.h"
 #include "parsecode.h"
 #include "parsing.h"
-#include "parser.h"
-#include "lex.yy.h"
+#include "query.h"
+#include "regulex.h"
+#include "shape.h"
+#include "tools.h"
+#include "unicode.h"
 
+#include "parser.h"
+
+#include "lex.yy.h"
 
 
 static pcre2_code *eol_pattern = NULL;
@@ -52,20 +58,18 @@ static int check_sizes(pass_to_bison *bison_args)
 {
     int i, j, k;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: check_sizes()\n");
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: check_sizes()\n");
 
-    for (i=0; i<NUM_SIDES; ++i) {
+    for (i = 0; i < NUM_SIDES; ++i) {
         if (i == 0 || i == 2) {
             /*
              *  horizontal
              */
-            for (j=0; j<SHAPES_PER_SIDE-1; ++j) {
+            for (j = 0; j < SHAPES_PER_SIDE - 1; ++j) {
                 if (curdes.shape[sides[i][j]].height == 0) {
                     continue;
                 }
-                for (k=j+1; k<SHAPES_PER_SIDE; ++k) {
+                for (k = j + 1; k < SHAPES_PER_SIDE; ++k) {
                     if (curdes.shape[sides[i][k]].height == 0) {
                         continue;
                     }
@@ -83,11 +87,11 @@ static int check_sizes(pass_to_bison *bison_args)
             /*
              *  vertical
              */
-            for (j=0; j<SHAPES_PER_SIDE-1; ++j) {
+            for (j = 0; j < SHAPES_PER_SIDE - 1; ++j) {
                 if (curdes.shape[sides[i][j]].width == 0) {
                     continue;
                 }
-                for (k=j+1; k<SHAPES_PER_SIDE; ++k) {
+                for (k = j + 1; k < SHAPES_PER_SIDE; ++k) {
                     if (curdes.shape[sides[i][k]].width == 0) {
                         continue;
                     }
@@ -103,7 +107,7 @@ static int check_sizes(pass_to_bison *bison_args)
         }
     }
 
-    return 0;                            /* all clear */
+    return 0; /* all clear */
 }
 
 
@@ -120,18 +124,16 @@ static int corner_check(pass_to_bison *bison_args)
 {
     int c;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: corner_check()\n");
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: corner_check()\n");
 
-    for (c=0; c<NUM_CORNERS; ++c) {
+    for (c = 0; c < NUM_CORNERS; ++c) {
         if (curdes.shape[corners[c]].elastic) {
             yyerror(bison_args, "Corners may not be elastic (%s)", shape_name[corners[c]]);
             return 1;
         }
     }
 
-    return 0;                            /* all clear */
+    return 0; /* all clear */
 }
 
 
@@ -140,17 +142,15 @@ static shape_t non_existent_elastics(pass_to_bison *bison_args)
 {
     shape_t i;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: non_existent_elastics()\n");
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: non_existent_elastics()\n");
 
-    for (i=0; i<NUM_SHAPES; ++i) {
-        if (curdes.shape[i].elastic && isempty(curdes.shape+i)) {
+    for (i = 0; i < NUM_SHAPES; ++i) {
+        if (curdes.shape[i].elastic && isempty(curdes.shape + i)) {
             return i;
         }
     }
 
-    return (shape_t) NUM_SHAPES;         /* all elastic shapes exist */
+    return (shape_t) NUM_SHAPES; /* all elastic shapes exist */
 }
 
 
@@ -159,22 +159,20 @@ static int insufficient_elasticity(pass_to_bison *bison_args)
 {
     int i, j, ef;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: insufficient_elasticity()\n");
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: insufficient_elasticity()\n");
 
-    for (i=0; i<NUM_SIDES; ++i) {
-        for (j=1,ef=0; j<4; ++j) {
+    for (i = 0; i < NUM_SIDES; ++i) {
+        for (j = 1, ef = 0; j < 4; ++j) {
             if (curdes.shape[sides[i][j]].elastic) {
                 ++ef;
             }
         }
         if (ef != 1 && ef != 2) {
-            return 1;                    /* error */
+            return 1; /* error */
         }
     }
 
-    return 0;                            /* all clear */
+    return 0; /* all clear */
 }
 
 
@@ -183,20 +181,19 @@ static int adjoining_elastics(pass_to_bison *bison_args)
 {
     int i, j, ef;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: adjoining_elastics()\n");
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: adjoining_elastics()\n");
 
-    for (i=0; i<NUM_SIDES; ++i) {
+    for (i = 0; i < NUM_SIDES; ++i) {
         ef = 0;
-        for (j=1; j<4; ++j) {
-            if (isempty(curdes.shape+sides[i][j])) {
+        for (j = 1; j < 4; ++j) {
+            if (isempty(curdes.shape + sides[i][j])) {
                 continue;
             }
             if (curdes.shape[sides[i][j]].elastic) {
                 if (ef) {
-                    return 1;            /* error detected */
-                } else {
+                    return 1; /* error detected */
+                }
+                else {
                     ef = 1;
                 }
             }
@@ -206,7 +203,7 @@ static int adjoining_elastics(pass_to_bison *bison_args)
         }
     }
 
-    return 0;                            /* all clear */
+    return 0; /* all clear */
 }
 
 
@@ -217,8 +214,7 @@ int perform_se_check(pass_to_bison *bison_args)
 
     s_rc = non_existent_elastics(bison_args);
     if (s_rc != NUM_SHAPES) {
-        yyerror(bison_args, "Shape %s has been specified as elastic but doesn't exist",
-                shape_name[s_rc]);
+        yyerror(bison_args, "Shape %s has been specified as elastic but doesn't exist", shape_name[s_rc]);
         return 1;
     }
 
@@ -244,7 +240,7 @@ int perform_se_check(pass_to_bison *bison_args)
 
 static void init_design(pass_to_bison *bison_args, design_t *design)
 {
-    memset (design, 0, sizeof(design_t));
+    memset(design, 0, sizeof(design_t));
     design->aliases = (char **) calloc(1, sizeof(char *));
     design->indentmode = DEF_INDENTMODE;
     design->defined_in = bison_args->config_file;
@@ -259,23 +255,20 @@ static void init_design(pass_to_bison *bison_args, design_t *design)
  */
 void recover(pass_to_bison *bison_args)
 {
-     bison_args->num_mandatory = 0;
-     bison_args->time_for_se_check = 0;
-     bison_args->num_shapespec = 0;
+    bison_args->num_mandatory = 0;
+    bison_args->time_for_se_check = 0;
+    bison_args->num_shapespec = 0;
 
-     /*
-      *  Clear current design
-      */
-     BFREE (curdes.name);
-     BFREE (curdes.author);
-     BFREE (curdes.aliases);
-     BFREE (curdes.designer);
-     BFREE (curdes.created);
-     BFREE (curdes.revision);
-     BFREE (curdes.revdate);
-     BFREE (curdes.sample);
-     BFREE (curdes.tags);
-     init_design(bison_args, &(curdes));
+    /*
+     *  Clear current design
+     */
+    BFREE(curdes.name);
+    BFREE(curdes.author);
+    BFREE(curdes.aliases);
+    BFREE(curdes.designer);
+    BFREE(curdes.sample);
+    BFREE(curdes.tags);
+    init_design(bison_args, &(curdes));
 }
 
 
@@ -312,11 +305,9 @@ static int full_parse_required()
 {
     int result = 0;
     if (!opt.design_choice_by_user) {
-        result = opt.r || opt.l || (opt.query != NULL && !query_is_undoc());
+        result = opt.r || opt.l || (opt.query != NULL && !opt.qundoc);
     }
-    #ifdef DEBUG
-        fprintf(stderr, " Parser: full_parse_required() -> %s\n", result ? "true" : "false");
-    #endif
+    log_debug(__FILE__, MAIN, " Parser: full_parse_required() -> %s\n", result ? "true" : "false");
     return result;
 }
 
@@ -370,23 +361,23 @@ static int alias_exists_in_child_configs(pass_to_bison *bison_args, char *alias)
 
 
 
-static int tag_add(pass_to_bison *bison_args, char *tag)
+static int tag_add(pass_to_bison *bison_args, bxstr_t *tag)
 {
     int rc = RC_SUCCESS;
-    if (tag_is_valid(tag)) {
-        if (!array_contains0(curdes.tags, tag)) {
+    if (is_ascii_id(tag, 1)) {
+        if (!array_contains0(curdes.tags, tag->ascii)) {
             size_t num_tags = array_count0(curdes.tags);
             curdes.tags = (char **) realloc(curdes.tags, (num_tags + 2) * sizeof(char *));
-            curdes.tags[num_tags] = tag;
+            curdes.tags[num_tags] = (char *) strdup(tag->ascii);
             curdes.tags[num_tags + 1] = NULL;
         }
         else {
-            yyerror(bison_args, "duplicate tag -- %s", tag);
+            yyerror(bison_args, "duplicate tag -- %s", bxs_to_output(tag));
             rc = RC_ERROR;
         }
     }
     else {
-        yyerror(bison_args, "invalid tag -- %s", tag);
+        yyerror(bison_args, "invalid tag -- %s", bxs_to_output(tag));
         rc = RC_ERROR;
     }
     return rc;
@@ -394,51 +385,41 @@ static int tag_add(pass_to_bison *bison_args, char *tag)
 
 
 
-static char *tag_next_comma(char *s)
-{
-    char *comma = strchr(s, ',');
-    if (comma == NULL) {
-        comma = s + strlen(s);
-    }
-    return comma;
-}
-
-
-
-static int tag_split_add(pass_to_bison *bison_args, char *tag)
+static int tag_split_add(pass_to_bison *bison_args, bxstr_t *tag)
 {
     int rc = RC_SUCCESS;
-    char *s = tag;
-    char *c = NULL;
+    uint32_t *c = NULL;
+    int vis_start = 0;
+    int cursor = -1;
     do {
-        c = tag_next_comma(s);
-        char *single_tag = trimdup(s, c - 1);
+        c = bxs_strchr(tag, to_utf32(','), &cursor);
+        bxstr_t *single_tag = bxs_trimdup(tag, vis_start, c != NULL ? (size_t) cursor : tag->num_chars_visible);
         int rc_add = tag_add(bison_args, single_tag);
+        bxs_free(single_tag);
         if (rc_add != 0) {
             rc = rc_add;
         }
-        s = c + 1;
-    } while (*c != '\0');
+        vis_start = cursor + 1;
+    } while (c != NULL);
     return rc;
 }
 
 
 
-/**
- * Add tag to list of current design's tag after validity checking.
- * @param bison_args the parser state
- * @param tag a single tag, or a comma-separated list of tags
- * @return error code, 0 on success, anything else on failure
- */
-int tag_record(pass_to_bison *bison_args, char *tag)
+int tag_record(pass_to_bison *bison_args, bxstr_t *tag)
 {
     int rc = RC_SUCCESS;
-    if (strchr(tag, ',') != NULL) {
+    if (tag->num_chars_invisible > 0) {
+        yyerror(bison_args, "invalid tag");
+        return RC_ERROR;
+    }
+    if (bxs_strchr(tag, to_utf32(','), NULL) != NULL) {
         rc = tag_split_add(bison_args, tag);
     }
     else {
-        char *trimmed = trimdup(tag, tag + strlen(tag) - 1);
+        bxstr_t *trimmed = bxs_trimdup(tag, 0, tag->num_chars_visible);
         rc = tag_add(bison_args, trimmed);
+        bxs_free(trimmed);
     }
     return rc;
 }
@@ -454,12 +435,12 @@ int tag_record(pass_to_bison *bison_args, char *tag)
  */
 int action_finalize_shapes(pass_to_bison *bison_args)
 {
-    int     i,j;
-    shape_t fshape;                  /* found shape */
-    int     fside;                   /* first side */
-    int     sc;                      /* side counter */
-    int     side;                    /* effective side */
-    int     rc;                      /* received return code */
+    int i, j;
+    shape_t fshape; /* found shape */
+    int fside;      /* first side */
+    int sc;         /* side counter */
+    int side;       /* effective side */
+    int rc;         /* received return code */
 
     /*
      *  At least one shape must be specified
@@ -474,28 +455,29 @@ int action_finalize_shapes(pass_to_bison *bison_args)
      *  as necessary, starting at any side which already includes at
      *  least one shape in order to ensure correct measurements.
      */
-    fshape = findshape (curdes.shape, NUM_SHAPES);
+    fshape = findshape(curdes.shape, NUM_SHAPES);
     if (fshape == NUM_SHAPES) {
         yyerror(bison_args, "internal error");
         return RC_ABORT;
     }
-    fside = on_side (fshape, 0);
+    fside = on_side(fshape, 0);
     if (fside == NUM_SIDES) {
         yyerror(bison_args, "internal error");
         return RC_ABORT;
     }
 
-    for (sc=0,side=fside; sc<NUM_SIDES; ++sc,side=(side+1)%NUM_SIDES) {
-        shape_t   nshape;               /* next shape */
-        sentry_t *c;                    /* corner to be processed */
-        c = curdes.shape + sides[side][SHAPES_PER_SIDE-1];
+    for (sc = 0, side = fside; sc < NUM_SIDES; ++sc, side = (side + 1) % NUM_SIDES) {
+        shape_t nshape; /* next shape */
+        sentry_t *c;    /* corner to be processed */
+        c = curdes.shape + sides[side][SHAPES_PER_SIDE - 1];
 
         if (isempty(c)) {
-            nshape = findshape (c, SHAPES_PER_SIDE);
+            nshape = findshape(c, SHAPES_PER_SIDE);
             if (side == BLEF || side == BRIG) {
                 if (nshape == SHAPES_PER_SIDE) {
                     c->height = 1;
-                } else {
+                }
+                else {
                     c->height = c[nshape].height;
                 }
                 c->width = curdes.shape[fshape].width;
@@ -503,36 +485,38 @@ int action_finalize_shapes(pass_to_bison *bison_args)
             else {
                 if (nshape == SHAPES_PER_SIDE) {
                     c->width = 1;
-                } else {
+                }
+                else {
                     c->width = c[nshape].width;
                 }
                 c->height = curdes.shape[fshape].height;
             }
             c->elastic = 0;
-            rc = genshape (c->width, c->height, &(c->chars));
+            rc = genshape(c->width, c->height, &(c->chars), &(c->mbcs));
             if (rc) {
                 return RC_ABORT;
             }
         }
 
-        fshape = sides[side][SHAPES_PER_SIDE-1];
+        fshape = sides[side][SHAPES_PER_SIDE - 1];
     }
 
     /*
      *  For all sides whose side shapes have not been defined, generate
      *  an elastic middle side shape.
      */
-    for (side=0; side<NUM_SIDES; ++side) {
+    for (side = 0; side < NUM_SIDES; ++side) {
         int found = 0;
-        for (i=1; i<SHAPES_PER_SIDE-1; ++i) {
-            if (isempty (curdes.shape + sides[side][i])) {
+        for (i = 1; i < SHAPES_PER_SIDE - 1; ++i) {
+            if (isempty(curdes.shape + sides[side][i])) {
                 continue;
-            } else {
+            }
+            else {
                 found = 1;
             }
         }
         if (!found) {
-            sentry_t *c = curdes.shape + sides[side][SHAPES_PER_SIDE/2];
+            sentry_t *c = curdes.shape + sides[side][SHAPES_PER_SIDE / 2];
             if (side == BLEF || side == BRIG) {
                 c->width = curdes.shape[sides[side][0]].width;
                 c->height = 1;
@@ -542,7 +526,7 @@ int action_finalize_shapes(pass_to_bison *bison_args)
                 c->height = curdes.shape[sides[side][0]].height;
             }
             c->elastic = 1;
-            rc = genshape (c->width, c->height, &(c->chars));
+            rc = genshape(c->width, c->height, &(c->chars), &(c->mbcs));
             if (rc) {
                 return RC_ABORT;
             }
@@ -563,10 +547,10 @@ int action_finalize_shapes(pass_to_bison *bison_args)
     /*
      *  Compute minimum height/width of a box of current design
      */
-    for (i=0; i<NUM_SIDES; ++i) {
+    for (i = 0; i < NUM_SIDES; ++i) {
         size_t c = 0;
-        if (i % 2) {                 /* vertical sides */
-            for (j=0; j<SHAPES_PER_SIDE; ++j) {
+        if (i % 2) { /* vertical sides */
+            for (j = 0; j < SHAPES_PER_SIDE; ++j) {
                 if (!isempty(curdes.shape + sides[i][j])) {
                     c += curdes.shape[sides[i][j]].height;
                 }
@@ -575,8 +559,8 @@ int action_finalize_shapes(pass_to_bison *bison_args)
                 curdes.minheight = c;
             }
         }
-        else {                       /* horizontal sides */
-            for (j=0; j<SHAPES_PER_SIDE; ++j) {
+        else { /* horizontal sides */
+            for (j = 0; j < SHAPES_PER_SIDE; ++j) {
                 if (!isempty(curdes.shape + sides[i][j])) {
                     c += curdes.shape[sides[i][j]].width;
                 }
@@ -590,7 +574,7 @@ int action_finalize_shapes(pass_to_bison *bison_args)
     /*
      *  Compute height of highest shape in design
      */
-    for (i=0; i<NUM_SHAPES; ++i) {
+    for (i = 0; i < NUM_SHAPES; ++i) {
         if (isempty(curdes.shape + i)) {
             continue;
         }
@@ -598,12 +582,16 @@ int action_finalize_shapes(pass_to_bison *bison_args)
             curdes.maxshapeheight = curdes.shape[i].height;
         }
     }
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: Minimum box dimensions: width %d height %d\n",
+    log_debug(__FILE__, PARSER, " Parser: Minimum box dimensions: width %d height %d\n",
                 (int) curdes.minwidth, (int) curdes.minheight);
-        fprintf (stderr, " Parser: Maximum shape height: %d\n",
-                (int) curdes.maxshapeheight);
-    #endif
+    log_debug(__FILE__, PARSER, " Parser: Maximum shape height: %d\n", (int) curdes.maxshapeheight);
+
+    /*
+     *  Set name of each shape
+     */
+    for (i = 0; i < NUM_SHAPES; ++i) {
+        curdes.shape[i].name = i;
+    }
     return RC_SUCCESS;
 }
 
@@ -630,93 +618,80 @@ int action_start_parsing_design(pass_to_bison *bison_args, char *design_name)
 
     if (!design_needed(bison_args)) {
         bison_args->speeding = 1;
-        #ifdef PARSER_DEBUG
-            fprintf (stderr, " Parser: Skipping to next design (lexer doesn't know!)\n");
-        #endif
-        return RC_ERROR;         /* trigger the parser's `error` rule, which will skip to the next design */
+        log_debug(__FILE__, PARSER, " Parser: Skipping to next design (lexer doesn't know!)\n");
+        return RC_ERROR; /* trigger the parser's `error` rule, which will skip to the next design */
     }
     return RC_SUCCESS;
 }
 
 
 
-/**
- * Rule action called when a parent config file specification is encountered.
- * @param bison_args the parser state
- * @param filepath the path to the parent config file
- * @return 0: success;
- *         1: YYERROR must be invoked
- */
-int action_parent_config(pass_to_bison *bison_args, char *filepath)
+int action_parent_config(pass_to_bison *bison_args, bxstr_t *filepath)
 {
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: parent config file specified: [%s]\n", filepath);
-    #endif
-    if (filepath == NULL || filepath[0] == '\0') {
+    if (is_debug_logging(PARSER)) {
+        char *out_filepath = bxs_to_output(filepath);
+        log_debug(__FILE__, PARSER, " Parser: parent config file specified: [%s]\n", out_filepath);
+        BFREE(out_filepath);
+    }
+
+    if (bxs_is_empty(filepath)) {
         bison_args->skipping = 1;
         yyerror(bison_args, "parent reference is empty");
         return RC_ERROR;
     }
-    else if (strcasecmp(filepath, ":global:") == 0) {    /* special token */
+    else if (strcasecmp(filepath->ascii, ":global:") == 0) { /* special token */
         filepath = discover_config_file(1);
         if (filepath == NULL) {
-            bison_args->skipping = 1;   /* prevent redundant "skipping to next design" message */
+            bison_args->skipping = 1; /* prevent redundant "skipping to next design" message */
             yyerror(bison_args, "parent reference to global config which cannot be found");
             return RC_ERROR;
         }
     }
+    else if (!bxs_valid_in_filename(filepath, NULL)) {
+        yyerror(bison_args, "parent reference contains invalid characters: %s", bxs_to_output(filepath));
+        return RC_ERROR;
+    }
     else {
-        FILE *f = fopen(filepath, "r");
+        FILE *f = bx_fopens(filepath, "r");
         if (f == NULL) {
             bison_args->skipping = 1;
-            yyerror(bison_args, "parent config file not found: %s", filepath);
+            yyerror(bison_args, "parent config file not found: %s", bxs_to_output(filepath));
             return RC_ERROR;
         }
         else {
             fclose(f);
         }
     }
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: parent config file path resolved: [%s]\n", filepath);
-    #endif
-    
-    int is_new = !array_contains(bison_args->parent_configs, bison_args->num_parent_configs, filepath);
-    if (!is_new) {
-        #ifdef PARSER_DEBUG
-            fprintf (stderr, " Parser: duplicate parent / cycle: [%s]\n", filepath);
-        #endif
+    if (is_debug_logging(PARSER)) {
+        char *out_filepath = bxs_to_output(filepath);
+        log_debug(__FILE__, PARSER, " Parser: parent config file path resolved: [%s]\n", out_filepath);
+        BFREE(out_filepath);
     }
-    else {
-        bison_args->parent_configs = realloc(bison_args->parent_configs,
-                (bison_args->num_parent_configs + 1) * sizeof(char *));
+
+    int is_new = !array_contains_bxs(bison_args->parent_configs, bison_args->num_parent_configs, filepath);
+    if (is_new) {
+        bison_args->parent_configs
+                = realloc(bison_args->parent_configs, (bison_args->num_parent_configs + 1) * sizeof(bxstr_t *));
         bison_args->parent_configs[bison_args->num_parent_configs] = filepath;
         ++(bison_args->num_parent_configs);
     }
+    else if (is_debug_logging(PARSER)) {
+        char *out_filepath = bxs_to_output(filepath);
+        log_debug(__FILE__, PARSER, " Parser: duplicate parent / cycle: [%s]\n", out_filepath);
+        BFREE(out_filepath);
+    }
     return RC_SUCCESS;
 }
 
 
 
-/**
- * Rule action called when a design has been parsed completely and can be checked and added.
- * @param bison_args the parser state
- * @param design_primary_name the primary name of the design as specified at BOX statement
- * @param name_at_end the primary name of the design as specified at END statement
- * @return 0: success;
- *         1: YYERROR must be invoked
- *         2: YYABORT must be invoked
- *         3: YYACCEPT must be invoked
- */
 int action_add_design(pass_to_bison *bison_args, char *design_primary_name, char *name_at_end)
 {
     design_t *tmp;
-    char *p;
 
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, "--------- ADDING DESIGN \"%s\".\n", design_primary_name);
-    #endif
+    log_debug(__FILE__, PARSER, "--------- ADDING DESIGN \"%s\".\n", design_primary_name);
 
-    if (strcasecmp (design_primary_name, name_at_end)) {
+    if (strcasecmp(design_primary_name, name_at_end)) {
         yyerror(bison_args, "box design name differs at BOX and END");
         return RC_ERROR;
     }
@@ -729,15 +704,6 @@ int action_add_design(pass_to_bison *bison_args, char *design_primary_name, char
         return RC_ERROR;
     }
 
-    p = design_primary_name;
-    while (*p) {
-        if (*p < 32 || *p > 126) {
-            yyerror(bison_args, "box design name must consist of printable standard ASCII characters.");
-            return RC_ERROR;
-        }
-        ++p;
-    }
-
     bison_args->num_mandatory = 0;
     bison_args->time_for_se_check = 0;
     bison_args->num_shapespec = 0;
@@ -756,7 +722,7 @@ int action_add_design(pass_to_bison *bison_args, char *design_primary_name, char
     ++(bison_args->design_idx);
     tmp = (design_t *) realloc(bison_args->designs, (bison_args->design_idx + 1) * sizeof(design_t));
     if (tmp == NULL) {
-        perror (PROJECT);
+        perror(PROJECT);
         return RC_ABORT;
     }
     bison_args->designs = tmp;
@@ -767,63 +733,41 @@ int action_add_design(pass_to_bison *bison_args, char *design_primary_name, char
 
 
 
-/**
- * Rule action called when a keyword entry is encountered in a box design.
- * @param bison_args the parser state
- * @param keyword the keyword
- * @param value the associated value
- * @return 0: success;
- *         1: YYERROR must be invoked
- *         2: YYABORT must be invoked
- */
-int action_record_keyword(pass_to_bison *bison_args, char *keyword, char *value)
+int action_record_keyword(pass_to_bison *bison_args, char *keyword, bxstr_t *value)
 {
-    #ifdef PARSER_DEBUG
-        fprintf (stderr, " Parser: entry rule fulfilled [%s = %s]\n", keyword, value);
-    #endif
-    if (strcasecmp (keyword, "author") == 0) {
-        curdes.author = (char *) strdup (value);
+    if (is_debug_logging(PARSER)) {
+        char *out_value = bxs_to_output(value);
+        log_debug(__FILE__, PARSER, " Parser: entry rule fulfilled [%s = %s]\n", keyword, out_value);
+        BFREE(out_value);
+    }
+
+    size_t error_pos = 0;
+    if (!bxs_valid_in_kv_string(value, &error_pos)) {
+        yyerror(bison_args, "invalid character in string value at position %d", (int) error_pos);
+        return RC_ERROR;
+    }
+
+    if (strcasecmp(keyword, "author") == 0) {
+        curdes.author = bxs_strdup(value);
         if (curdes.author == NULL) {
-            perror (PROJECT);
+            perror(PROJECT);
             return RC_ABORT;
         }
     }
-    else if (strcasecmp (keyword, "designer") == 0) {
-        curdes.designer = (char *) strdup (value);
+    else if (strcasecmp(keyword, "designer") == 0) {
+        curdes.designer = bxs_strdup(value);
         if (curdes.designer == NULL) {
-            perror (PROJECT);
+            perror(PROJECT);
             return RC_ABORT;
         }
     }
-    else if (strcasecmp (keyword, "revision") == 0) {
-        curdes.revision = (char *) strdup (value);
-        if (curdes.revision == NULL) {
-            perror (PROJECT);
-            return RC_ABORT;
-        }
+    else if (strcasecmp(keyword, "tags") == 0) {
+        tag_record(bison_args, value); /* discard return code (we print warnings, but tolerate the problem) */
     }
-    else if (strcasecmp (keyword, "created") == 0) {
-        curdes.created = (char *) strdup (value);
-        if (curdes.created == NULL) {
-            perror (PROJECT);
-            return RC_ABORT;
-        }
-    }
-    else if (strcasecmp (keyword, "revdate") == 0) {
-        curdes.revdate = (char *) strdup (value);
-        if (curdes.revdate == NULL) {
-            perror (PROJECT);
-            return RC_ABORT;
-        }
-    }
-    else if (strcasecmp (keyword, "tags") == 0) {
-        tag_record(bison_args, value);    /* discard return code (we print warnings, but tolerate the problem) */
-    }
-    else if (strcasecmp (keyword, "indent") == 0) {
-        if (strcasecmp (value, "text") == 0 ||
-            strcasecmp (value, "box") == 0 ||
-            strcasecmp (value, "none") == 0) {
-            curdes.indentmode = value[0];
+    else if (strcasecmp(keyword, "indent") == 0) {
+        char *val = value->ascii;
+        if (strcasecmp(val, "text") == 0 || strcasecmp(val, "box") == 0 || strcasecmp(val, "none") == 0) {
+            curdes.indentmode = val[0];
         }
         else {
             yyerror(bison_args, "indent keyword must be followed by \"text\", \"box\", or \"none\"");
@@ -883,9 +827,9 @@ int action_padding_entry(pass_to_bison *bison_args, char *area, int value)
 
 int action_init_parser(pass_to_bison *bison_args)
 {
-    bison_args->designs = (design_t *) calloc (1, sizeof(design_t));
+    bison_args->designs = (design_t *) calloc(1, sizeof(design_t));
     if (bison_args->designs == NULL) {
-        perror (PROJECT);
+        perror(PROJECT);
         return RC_ABORT;
     }
     bison_args->num_designs = 1;
@@ -902,9 +846,7 @@ int action_add_alias(pass_to_bison *bison_args, char *alias_name)
         return RC_ERROR;
     }
     if (alias_exists_in_child_configs(bison_args, alias_name)) {
-        #ifdef PARSER_DEBUG
-            fprintf (stderr, " Parser: alias already used by child config, dropping: %s\n", alias_name);
-        #endif
+        log_debug(__FILE__, PARSER, " Parser: alias already used by child config, dropping: %s\n", alias_name);
     }
     else {
         size_t num_aliases = array_count0(curdes.aliases);
@@ -917,46 +859,193 @@ int action_add_alias(pass_to_bison *bison_args, char *alias_name)
 
 
 
-static char *adjust_eols(char *sample)
+static bxstr_t *adjust_eols(uint32_t *sample)
 {
     if (eol_pattern == NULL) {
-        eol_pattern = compile_pattern("(?(?=\r)(\r\n?)|(\n))");
+        eol_pattern = compile_pattern("(?:(\r\n?)|(\n))");
     }
-    uint32_t *u32_sample = u32_strconv_from_input(sample);
-    uint32_t *replaced = regex_replace(eol_pattern, opt.eol, u32_sample, strlen(sample), 1);
-    char *result = u32_strconv_to_output(replaced);
+    uint32_t *replaced = regex_replace(eol_pattern, opt.eol, sample, u32_strlen(sample), 1);
+    bxstr_t *result = bxs_from_unicode(replaced);
     BFREE(replaced);
-    BFREE(u32_sample);
     return result;
 }
 
 
 
-int action_sample_block(pass_to_bison *bison_args, char *sample)
+static uint32_t *find_first_nonblank_line(bxstr_t *sample)
 {
-    #ifdef PARSER_DEBUG
-        fprintf(stderr, " Parser: SAMPLE block rule satisfied\n");
-    #endif
+    uint32_t *result = sample->memory;
+    uint32_t *p = sample->memory;
+    while (*p != char_nul) {
+        while (uc_is_blank(*p) || *p == char_cr) {
+            p++;
+        }
+        if (*p == char_newline) {
+            result = ++p;
+        }
+        else {
+            break;
+        }
+    }
+    return result;
+}
+
+
+
+int action_sample_block(pass_to_bison *bison_args, bxstr_t *sample)
+{
+    log_debug(__FILE__, PARSER, " Parser: SAMPLE block rule satisfied\n");
 
     if (curdes.sample) {
         yyerror(bison_args, "duplicate SAMPLE block");
         return RC_ERROR;
     }
 
-    char *p = sample;
-    while ((*p == '\r' || *p == '\n') && *p != '\0') {
-        p++;
-    }
-    char *line = adjust_eols(p);
+    uint32_t *p = find_first_nonblank_line(sample);
+    bxstr_t *line = adjust_eols(p);
     if (line == NULL) {
         perror(PROJECT);
         return RC_ABORT;
     }
 
+    if (bxs_is_empty(line)) {
+        yyerror(bison_args, "SAMPLE block must not be empty");
+        return RC_ERROR;
+    }
+    size_t error_pos = 0;
+    if (!bxs_valid_in_sample(line, &error_pos)) {
+        yyerror(bison_args, "invalid character in SAMPLE block at position %d", (int) error_pos);
+        return RC_ERROR;
+    }
+
     curdes.sample = line;
     ++(bison_args->num_mandatory);
     return RC_SUCCESS;
 }
 
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+
+int action_add_regex_rule(pass_to_bison *bison_args, char *type, reprule_t **rule_list, size_t *rule_list_len,
+        bxstr_t *search, bxstr_t *replace, char mode)
+{
+    size_t n = *rule_list_len;
+
+    if (is_debug_logging(PARSER)) {
+        char *out_search = bxs_to_output(search);
+        char *out_replace = bxs_to_output(replace);
+        log_debug(__FILE__, PARSER, "Adding %s rule: \"%s\" with \"%s\" (%c)\n",
+                strcmp(type, "rep") == 0 ? "replacement" : "reversion", out_search, out_replace, mode);
+        BFREE(out_replace);
+        BFREE(out_search);
+    }
+
+    *rule_list = (reprule_t *) realloc(*rule_list, (n + 1) * sizeof(reprule_t));
+    if (*rule_list == NULL) {
+        perror(PROJECT);
+        return RC_ABORT;
+    }
+    memset((*rule_list) + n, 0, sizeof(reprule_t));
+    (*rule_list)[n].search = bxs_strdup(search);
+    (*rule_list)[n].repstr = bxs_strdup(replace);
+    if ((*rule_list)[n].search == NULL || (*rule_list)[n].repstr == NULL) {
+        perror(PROJECT);
+        return RC_ABORT;
+    }
+    (*rule_list)[n].line = yyget_lineno(bison_args->lexer_state);
+    (*rule_list)[n].mode = mode;
+    *rule_list_len = n + 1;
+    return RC_SUCCESS;
+}
+
+
+
+int action_first_shape_line(pass_to_bison *bison_args, bxstr_t *line, sentry_t *shape)
+{
+    sentry_t rval = SENTRY_INITIALIZER;
+
+    log_debug(__FILE__, PARSER, "Initializing a shape entry with first line\n");
+
+    size_t error_pos = 0;
+    if (!bxs_valid_in_shape(line, &error_pos)) {
+        yyerror(bison_args, "invalid character in shape line at position %d", (int) error_pos);
+        return RC_ERROR;
+    }
+
+    rval.width = line->num_columns;
+    rval.height = 1;
+
+    rval.chars = (char **) malloc(sizeof(char *));
+    if (rval.chars == NULL) {
+        perror(PROJECT ": shape_lines21");
+        return RC_ABORT;
+    }
+    rval.chars[0] = (char *) strdup(line->ascii);
+    if (rval.chars[0] == NULL) {
+        perror(PROJECT ": shape_lines22");
+        return RC_ABORT;
+    }
+
+    rval.mbcs = (bxstr_t **) malloc(sizeof(bxstr_t *));
+    if (rval.mbcs == NULL) {
+        perror(PROJECT ": shape_lines23");
+        return RC_ABORT;
+    }
+    rval.mbcs[0] = bxs_strdup(line);
+    if (rval.mbcs[0] == NULL) {
+        perror(PROJECT ": shape_lines24");
+        return RC_ABORT;
+    }
+
+    memcpy(shape, &rval, sizeof(sentry_t));
+    return RC_SUCCESS;
+}
+
+
+
+int action_add_shape_line(pass_to_bison *bison_args, sentry_t *shape, bxstr_t *line)
+{
+    log_debug(__FILE__, PARSER, "Extending a shape entry\n");
+
+    size_t slen = line->num_columns;
+    if (slen != shape->width) {
+        yyerror(bison_args, "all elements of a shape spec must be of equal length");
+        return RC_ERROR;
+    }
+
+    size_t error_pos = 0;
+    if (!bxs_valid_in_shape(line, &error_pos)) {
+        yyerror(bison_args, "invalid character in shape line at position %d", (int) error_pos);
+        return RC_ERROR;
+    }
+
+    shape->height++;
+
+    char **tmp = (char **) realloc(shape->chars, shape->height * sizeof(char *));
+    if (tmp == NULL) {
+        perror(PROJECT ": shape_lines11");
+        return RC_ABORT;
+    }
+    shape->chars = tmp;
+    shape->chars[shape->height - 1] = (char *) strdup(line->ascii);
+    if (shape->chars[shape->height - 1] == NULL) {
+        perror(PROJECT ": shape_lines12");
+        return RC_ABORT;
+    }
+
+    bxstr_t **mtmp = (bxstr_t **) realloc(shape->mbcs, shape->height * sizeof(bxstr_t *));
+    if (mtmp == NULL) {
+        perror(PROJECT ": shape_lines13");
+        return RC_ABORT;
+    }
+    shape->mbcs = mtmp;
+    shape->mbcs[shape->height - 1] = bxs_strdup(line);
+    if (shape->mbcs[shape->height - 1] == NULL) {
+        perror(PROJECT ": shape_lines14");
+        return RC_ABORT;
+    }
+
+    return RC_SUCCESS;
+}
+
+
+/* vim: set cindent sw=4: */
