@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -29,13 +29,6 @@
 char **parse_query(char *optarg);
 
 
-/**
- * Check if -q "(undoc)" was specified.
- * @returns flag
- */
-int query_is_undoc();
-
-
 /**
  * Perform the tag query based on the global design list and the query from the global `opt` struct.
  * @returns 0 if successful; anything else on error (then the program should exit)
@@ -45,4 +38,4 @@ int query_by_tag();
 
 #endif
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
