@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,12 +18,16 @@
  */
 
 #include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
-#include "shape.h"
+
 #include "boxes.h"
+#include "bxstring.h"
+#include "logging.h"
+#include "shape.h"
 #include "tools.h"
 
 
@@ -38,6 +42,7 @@ shape_t north_side[SHAPES_PER_SIDE] = {NW, NNW, N, NNE, NE};  /* clockwise */
 shape_t east_side[SHAPES_PER_SIDE] = {NE, ENE, E, ESE, SE};
 
 shape_t south_side[SHAPES_PER_SIDE] = {SE, SSE, S, SSW, SW};
+shape_t south_side_rev[SHAPES_PER_SIDE] = {SW, SSW, S, SSE, SE};
 
 shape_t west_side[SHAPES_PER_SIDE] = {SW, WSW, W, WNW, NW};
 
@@ -109,13 +114,14 @@ int on_side(const shape_t s, const int idx)
 
 
 
-int genshape(const size_t width, const size_t height, char ***chars)
+int genshape(const size_t width, const size_t height, char ***chars, bxstr_t ***mbcs)
 /*
  *  Generate a shape consisting of spaces only.
  *
  *      width   desired shape width
  *      height  desired shape height
  *      chars   pointer to the shape lines (should be NULL upon call)
+ *      mbcs    pointer to the shape lines, MBCS version (should be NULL upon call)
  *
  *  Memory is allocated for the shape lines which must be freed by the caller.
  *
@@ -138,15 +144,16 @@ int genshape(const size_t width, const size_t height, char ***chars)
         return 2;
     }
 
+    *mbcs = (bxstr_t **) calloc(height, sizeof(bxstr_t *));
+    if (*mbcs == NULL) {
+        BFREE(*chars);
+        perror(PROJECT);
+        return 4;
+    }
+
     for (j = 0; j < height; ++j) {
-        (*chars)[j] = (char *) calloc(width + 1, sizeof(char));
-        if ((*chars)[j] == NULL) {
-            perror(PROJECT);
-            for (/*empty*/; j > 0; --j) BFREE ((*chars)[j - 1]);
-            BFREE (*chars);
-            return 3;
-        }
-        memset((*chars)[j], ' ', width);
+        (*chars)[j] = nspaces(width);
+        (*mbcs)[j] = bxs_from_ascii((*chars)[j]);
     }
 
     return 0;
@@ -166,8 +173,12 @@ void freeshape(sentry_t *shape)
 
     for (j = 0; j < shape->height; ++j) {
         BFREE (shape->chars[j]);
+        bxs_free(shape->mbcs[j]);
     }
     BFREE (shape->chars);
+    BFREE (shape->mbcs);
+    BFREE (shape->blank_leftward);
+    BFREE (shape->blank_rightward);
 
     *shape = SENTRY_INITIALIZER;
 }
@@ -183,7 +194,7 @@ int isempty(const sentry_t *shape)
 {
     if (shape == NULL) {
         return 1;
-    } else if (shape->chars == NULL) {
+    } else if (shape->chars == NULL || shape->mbcs == NULL) {
         return 1;
     } else if (shape->width == 0 || shape->height == 0) {
         return 1;
@@ -238,10 +249,6 @@ size_t highest(const sentry_t *sarr, const int n, ...)
     int i;
     size_t max = 0;                      /* current maximum height */
 
-    #if defined(DEBUG) && 0
-    fprintf (stderr, "highest (%d, ...)\n", n);
-    #endif
-
     va_start (ap, n);
 
     for (i = 0; i < n; ++i) {
@@ -277,10 +284,6 @@ size_t widest(const sentry_t *sarr, const int n, ...)
     int i;
     size_t max = 0;                      /* current maximum width */
 
-    #if defined(DEBUG) && 0
-    fprintf (stderr, "widest (%d, ...)\n", n);
-    #endif
-
     va_start (ap, n);
 
     for (i = 0; i < n; ++i) {
@@ -299,71 +302,16 @@ size_t widest(const sentry_t *sarr, const int n, ...)
 
 
 
-shape_t leftmost(const int aside, const int cnt)
-/*
- *  Return leftmost existing shape in specification for side aside
- *  (BTOP or BBOT), skipping cnt shapes. Corners are not considered.
+/**
+ * Return true if the shapes on the given side consist entirely of spaces - and spaces only, tabs are considered
+ * non-empty.
  *
- *  RETURNS:    shape       if shape was found
- *              NUM_SHAPES  on error (e.g. cnt too high)
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * @param sarr pointer to shape list of design to check
+ * @param aside the box side (one of `BTOP` etc.)
+ * @return == 0: side is not empty;
+ *         \!= 0: side is empty
  */
-{
-    int c = 0;
-    int s;
-
-    if (cnt < 0) {
-        return NUM_SHAPES;
-    }
-
-    if (aside == BTOP) {
-        s = 0;
-        do {
-            ++s;
-            while (s < SHAPES_PER_SIDE - 1 &&
-                    isempty(opt.design->shape + north_side[s])) {
-                        ++s;
-            }
-            if (s == SHAPES_PER_SIDE - 1) {
-                return NUM_SHAPES;
-            }
-        } while (c++ < cnt);
-        return north_side[s];
-    }
-
-    else if (aside == BBOT) {
-        s = SHAPES_PER_SIDE - 1;
-        do {
-            --s;
-            while (s && isempty(opt.design->shape + south_side[s])) {
-                --s;
-            }
-            if (!s) {
-                return NUM_SHAPES;
-            }
-        } while (c++ < cnt);
-        return south_side[s];
-    }
-
-    return NUM_SHAPES;
-}
-
-
-
 int empty_side(sentry_t *sarr, const int aside)
-/*
- *  Return true if the shapes on the given side consist entirely out of
- *  spaces - and spaces only, tabs are considered non-empty.
- *
- *      sarr    pointer to shape list of design to check
- *      aside   the box side (one of BTOP etc.)
- *
- *  RETURNS:  == 0   side is not empty
- *            != 0   side is empty
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
 {
     int i;
 
@@ -380,4 +328,201 @@ int empty_side(sentry_t *sarr, const int aside)
 
 
 
-/*EOF*/                                                 /* vim: set sw=4: */
+static int is_west(sentry_t *shape, int include_corners)
+{
+    size_t offset = include_corners ? 0 : 1;
+    for (size_t i = offset; i < SHAPES_PER_SIDE - offset; i++) {
+        if (west_side[i] == shape->name) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+
+
+static int is_east(sentry_t *shape, int include_corners)
+{
+    size_t offset = include_corners ? 0 : 1;
+    for (size_t i = offset; i < SHAPES_PER_SIDE - offset; i++) {
+        if (east_side[i] == shape->name) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+
+
+static int find_in_horiz(shape_t side[], shape_t shape_name)
+{
+    int result = -1;
+    for (size_t i = 0; i < SHAPES_PER_SIDE; i++) {
+        if (side[i] == shape_name) {
+            result = i;
+            break;
+        }
+    }
+    return result;
+}
+
+
+
+static int find_in_north(shape_t shape_name)
+{
+    return find_in_horiz(north_side, shape_name);
+}
+
+
+
+static int find_in_south(shape_t shape_name)
+{
+    return find_in_horiz(south_side_rev, shape_name);
+}
+
+
+
+static int *new_blankward_cache(const size_t shape_height)
+{
+    int *result = (int *) calloc(shape_height, sizeof(int));
+    for (size_t i = 0; i < shape_height; i++) {
+        result[i] = -1;
+    }
+    return result;
+}
+
+
+
+static int is_blankward_calc(
+        design_t *current_design, const shape_t shape, const size_t shape_line_idx, const int is_leftward)
+{
+    int result = -1;
+    sentry_t *shape_data = current_design->shape + shape;
+    if (is_west(shape_data, is_leftward ? 1 : 0)) {
+        result = is_leftward ? 1 : 0;
+    }
+    else if (is_east(shape_data, is_leftward ? 0 : 1)) {
+        result = is_leftward ? 0 : 1;
+    }
+    else {
+        shape_t *side = north_side;
+        int pos = find_in_north(shape);
+        if (pos < 0) {
+            side = south_side_rev;
+            pos = find_in_south(shape);
+        }
+        result = 1;
+        size_t loop_init = (size_t) pos + 1;
+        size_t loop_end = SHAPES_PER_SIDE;
+        if (is_leftward) {
+            loop_init = 0;
+            loop_end = (size_t) pos;
+        }
+        for (size_t i = loop_init; i < loop_end; i++) {
+            sentry_t *tshape = current_design->shape + side[i];
+            if (tshape->mbcs != NULL && !bxs_is_blank(tshape->mbcs[shape_line_idx])) {
+                result = 0;
+                break;
+            }
+        }
+    }
+    return result;
+}
+
+
+
+int is_blankward(design_t *current_design, const shape_t shape, const size_t shape_line_idx, const int is_leftward)
+{
+    if (current_design == NULL) {
+        return 0;  /* this would be a bug */
+    }
+    sentry_t *shape_data = current_design->shape + shape;
+    if (shape_line_idx >= shape_data->height) {
+        return 0;  /* this would be a bug */
+    }
+    int *blankward_cache = is_leftward ? shape_data->blank_leftward : shape_data->blank_rightward;
+    if (blankward_cache != NULL && blankward_cache[shape_line_idx] >= 0) {
+        return blankward_cache[shape_line_idx];  /* cached value available */
+    }
+    if (blankward_cache == NULL) {
+        blankward_cache = new_blankward_cache(shape_data->height);
+        if (is_leftward) {
+            shape_data->blank_leftward = blankward_cache;
+        }
+        else {
+            shape_data->blank_rightward = blankward_cache;
+        }
+    }
+
+    int result = is_blankward_calc(current_design, shape, shape_line_idx, is_leftward);
+    blankward_cache[shape_line_idx] = result;
+    return result;
+}
+
+
+
+void debug_print_shape(sentry_t *shape)
+{
+    if (is_debug_logging(MAIN)) {
+        if (shape == NULL) {
+            log_debug(__FILE__, MAIN, "NULL\n");
+            return;
+        }
+        log_debug(__FILE__, MAIN, "Shape %3s (%dx%d): elastic=%s,%s bl=",
+            shape_name[shape->name], (int) shape->width, (int) shape->height,
+            shape->elastic ? "true" : "false", shape->elastic ? " " : "");
+        if (shape->blank_leftward == NULL) {
+            log_debug_cont(MAIN, "NULL");
+        }
+        else {
+            log_debug_cont(MAIN, "[");
+            for (size_t i = 0; i < shape->height; i++) {
+                log_debug_cont(MAIN, "%d%s", shape->blank_leftward[i],
+                        shape->height > 0 && i < (shape->height - 1) ? ", " : "");
+            }
+            log_debug_cont(MAIN, "]");
+        }
+        log_debug_cont(MAIN, ", br=");
+        if (shape->blank_rightward == NULL) {
+            log_debug_cont(MAIN, "NULL");
+        }
+        else {
+            log_debug_cont(MAIN, "[");
+            for (size_t i = 0; i < shape->height; i++) {
+                log_debug_cont(MAIN, "%d%s", shape->blank_rightward[i],
+                        shape->height > 0 && i < (shape->height - 1) ? ", " : "");
+            }
+            log_debug_cont(MAIN, "]");
+        }
+        log_debug_cont(MAIN, ", ascii=");
+        if (shape->chars == NULL) {
+            log_debug_cont(MAIN, "NULL");
+        }
+        else {
+            log_debug_cont(MAIN, "[");
+            for (size_t i = 0; i < shape->height; i++) {
+                log_debug_cont(MAIN, "%s%s%s%s", shape->chars[i] != NULL ? "\"" : "", shape->chars[i],
+                        shape->chars[i] != NULL ? "\"" : "", (int) i < ((int) shape->height) - 1 ? ", " : "");
+            }
+            log_debug_cont(MAIN, "]");
+        }
+        log_debug_cont(MAIN, ", mbcs=");
+        if (shape->mbcs == NULL) {
+            log_debug_cont(MAIN, "NULL");
+        }
+        else {
+            log_debug_cont(MAIN, "[");
+            for (size_t i = 0; i < shape->height; i++) {
+                char *out_mbcs = bxs_to_output(shape->mbcs[i]);
+                log_debug_cont(MAIN, "%s%s%s%s", shape->mbcs[i] != NULL ? "\"" : "", out_mbcs,
+                        shape->mbcs[i] != NULL ? "\"" : "", shape->height > 0 && i < (shape->height - 1) ? ", " : "");
+                BFREE(out_mbcs);
+            }
+            log_debug_cont(MAIN, "]");
+        }
+        log_debug_cont(MAIN, "\n");
+    }
+}
+
+
+/* vim: set sw=4: */
