@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,22 +18,58 @@
  */
 
 #include "config.h"
+
 #include <errno.h>
-#include <stdio.h>
 #include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
-
+#include <uniconv.h>
 #include <unictype.h>
 #include <unistr.h>
 #include <unitypes.h>
 #include <uniwidth.h>
 
 #include "boxes.h"
+#include "logging.h"
+#include "regulex.h"
 #include "shape.h"
-#include "unicode.h"
 #include "tools.h"
+#include "unicode.h"
+
+
+
+static pcre2_code *pattern_ascii_id = NULL;
+static pcre2_code *pattern_ascii_id_strict = NULL;
+
+/**
+ * Initialize the `bx_fprintf` function pointer to point to the original
+ * `bx_fprintf` function, now renamed `bx_fprintf_original`. During unit
+ * tests, this will be replaced with `__wrap_bx_fprintf`, which stores
+ * the result that would have been printed so the output can be validated.
+ * This is necessary for unit testing and CI to work with MacOS.
+ */
+bx_fprintf_t bx_fprintf = bx_fprintf_original;
+
+
+static pcre2_code *get_pattern_ascii_id(int strict)
+{
+    pcre2_code *result = NULL;
+    if (strict) {
+        if (pattern_ascii_id_strict == NULL) {
+            pattern_ascii_id_strict = compile_pattern("^(?!.*?--|none)[a-z][a-z0-9-]*(?<!-)$");
+        }
+        result = pattern_ascii_id_strict;
+    }
+    else {
+        if (pattern_ascii_id == NULL) {
+            pattern_ascii_id = compile_pattern("^(?!.*?[-_]{2,}|none)[a-zA-Z][a-zA-Z0-9_-]*(?<![-_])$");
+        }
+        result = pattern_ascii_id;
+    }
+    return result;
+}
 
 
 
@@ -55,15 +91,20 @@ int strisyes(const char *s)
 
     if (!strncasecmp("on", s, 3)) {
         return 1;
-    } else if (!strncasecmp("yes", s, 4)) {
+    }
+    else if (!strncasecmp("yes", s, 4)) {
         return 1;
-    } else if (!strncasecmp("true", s, 5)) {
+    }
+    else if (!strncasecmp("true", s, 5)) {
         return 1;
-    } else if (!strncmp("1", s, 2)) {
+    }
+    else if (!strncmp("1", s, 2)) {
         return 1;
-    } else if (!strncasecmp("t", s, 2)) {
+    }
+    else if (!strncasecmp("t", s, 2)) {
         return 1;
-    } else {
+    }
+    else {
         return 0;
     }
 }
@@ -88,87 +129,26 @@ int strisno(const char *s)
 
     if (!strncasecmp("off", s, 4)) {
         return 1;
-    } else if (!strncasecmp("no", s, 3)) {
+    }
+    else if (!strncasecmp("no", s, 3)) {
         return 1;
-    } else if (!strncasecmp("false", s, 6)) {
+    }
+    else if (!strncasecmp("false", s, 6)) {
         return 1;
-    } else if (!strncmp("0", s, 2)) {
+    }
+    else if (!strncmp("0", s, 2)) {
         return 1;
-    } else if (!strncasecmp("f", s, 2)) {
+    }
+    else if (!strncasecmp("f", s, 2)) {
         return 1;
-    } else {
+    }
+    else {
         return 0;
     }
 }
 
 
 
-void concat_strings(char *dst, int max_len, int count, ...)
-/*
- *  Concatenate a variable number of strings into a fixed-length buffer.
- *
- *      dst     Destination array
- *      max_len Maximum resulting string length (including terminating NULL).
- *      count   Number of source strings.
- *
- *  The concatenation process terminates when either the destination
- *  buffer is full or all 'count' strings are processed.  Null string
- *  pointers are treated as empty strings.
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
-{
-    va_list va;
-    const char *src;
-
-    va_start (va, count);
-
-    /*
-     *  Sanity check.
-     */
-    if (max_len < 1) {
-        return;
-    }
-
-    if (max_len == 1 || count < 1) {
-        *dst = '\0';
-        return;
-    }
-
-    /*
-     *  Loop over all input strings.
-     */
-    while (count-- > 0 && max_len > 1) {
-
-        /*
-         * Grab an input string pointer.  If it's NULL, skip it (eg. treat
-         * it as empty.
-         */
-        src = va_arg (va, const char *);
-
-        if (src == NULL) {
-            continue;
-        }
-
-        /*
-         * Concatenate 'src' onto 'dst', as long as we have room.
-         */
-        while (*src && max_len > 1) {
-            *dst++ = *src++;
-            max_len--;
-        }
-    }
-
-    va_end (va);
-
-    /*
-     * Terminate the string with an ASCII NUL.
-     */
-    *dst = '\0';
-}
-
-
-
 char *concat_strings_alloc(size_t count, ...)
 {
     if (count < 1) {
@@ -179,27 +159,27 @@ char *concat_strings_alloc(size_t count, ...)
     const char *src;
     va_list va;
 
-    va_start (va, count);
+    va_start(va, count);
     for (size_t i = 0; i < count; i++) {
-        src = va_arg (va, const char *);
+        src = va_arg(va, const char *);
         if (src != NULL) {
             total_len += strlen(src);
         }
     }
-    va_end (va);
+    va_end(va);
 
     char *result = malloc(total_len + 1);
     char *p = result;
 
-    va_start (va, count);
+    va_start(va, count);
     for (size_t i = 0; i < count; i++) {
-        src = va_arg (va, const char *);
+        src = va_arg(va, const char *);
         if (src != NULL && src[0] != '\0') {
             strcpy(p, src);
             p += strlen(src);
         }
     }
-    va_end (va);
+    va_end(va);
 
     *p = '\0';
     return result;
@@ -207,59 +187,43 @@ char *concat_strings_alloc(size_t count, ...)
 
 
 
-int empty_line(const line_t *line)
-/*
- *  Return true if line is empty.
- *
- *  Empty lines either consist entirely of whitespace or don't exist.
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
+char *repeat(char *s, size_t count)
 {
-    char *p;
-    size_t j;
-
-    if (!line) {
-        return 1;
-    }
-    if (line->text == NULL || line->len <= 0) {
-        return 1;
+    if (s == NULL) {
+        return NULL;
     }
 
-    for (p = line->text, j = 0; *p && j < line->len; ++j, ++p) {
-        if (*p != ' ' && *p != '\t' && *p != '\r') {
-            return 0;
+    size_t len = strlen(s);
+    char *result = (char *) malloc(count * len + 1);
+    if (result != NULL) {
+        char *dest = result;
+        for (size_t i = 0; i < count; i++) {
+            strcpy(dest, s);
+            dest += len;
         }
+        *dest = '\0';
     }
-    return 1;
+    return result;
 }
 
 
 
-size_t expand_tabs_into(const uint32_t *input_buffer, const int tabstop, uint32_t **text,
-                        size_t **tabpos, size_t *tabpos_len)
-/*
- *  Expand tab chars in input_buffer and store result in text.
- *
- *  input_buffer   Line of text with tab chars
- *  tabstop        tab stop distance
- *  text           address of the pointer that will take the result
- *  tabpos         array of ints giving the positions of the first
- *                 space of an expanded tab in the text result buffer
- *  tabpos_len     number of tabs recorded in tabpos
- *
- *  Memory will be allocated for text and tabpos.
- *  Should only be called for lines of length > 0;
- *
- *  RETURNS:  Success: Length of the result line in characters (> 0)
- *            Error:   0       (e.g. out of memory)
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
+int empty_line(const line_t *line)
 {
-    static uint32_t temp[LINE_MAX_BYTES + 100];  /* work string */
-    size_t io;         /* character position in work string */
-    size_t tabnum = 0; /* index of the current tab */
+    if (!line) {
+        return 1;
+    }
+    return bxs_is_blank(line->text);
+}
+
+
+
+size_t expand_tabs_into(const uint32_t *input_buffer, const int tabstop, uint32_t **text, size_t **tabpos,
+        size_t *tabpos_len)
+{
+    static uint32_t temp[LINE_MAX_BYTES + 100]; /* work string */
+    size_t io;                                  /* character position in work string */
+    size_t tabnum = 0;                          /* index of the current tab */
 
     *text = NULL;
     *tabpos = NULL;
@@ -281,7 +245,7 @@ size_t expand_tabs_into(const uint32_t *input_buffer, const int tabstop, uint32_
     if (*tabpos_len > 0) {
         *tabpos = (size_t *) calloc((*tabpos_len) + 1, sizeof(size_t));
         if (*tabpos == NULL) {
-            return 0;       /* out of memory */
+            return 0; /* out of memory */
         }
     }
 
@@ -322,9 +286,7 @@ void btrim(char *text, size_t *len)
 {
     long idx = (long) (*len) - 1;
 
-    while (idx >= 0 && (text[idx] == '\n' || text[idx] == '\r'
-            || text[idx] == '\t' || text[idx] == ' ')) /**/
-    {
+    while (idx >= 0 && (text[idx] == '\n' || text[idx] == '\r' || text[idx] == '\t' || text[idx] == ' ')) {
         text[idx--] = '\0';
     }
 
@@ -336,6 +298,7 @@ void btrim(char *text, size_t *len)
 void btrim32(uint32_t *text, size_t *len)
 /*
  *  Remove trailing whitespace from line (unicode and escape sequence enabled version).
+ *  CHECK replace by bxs_rtrim() from bxstring module
  *
  *      text     string to trim
  *      len      pointer to the length of the string in characters
@@ -377,78 +340,30 @@ void btrim32(uint32_t *text, size_t *len)
 
 
 
-char *my_strnrstr(const char *s1, const char *s2, const size_t s2_len, int skip)
-/*
- *  Return pointer to last occurrence of string s2 in string s1.
- *
- *      s1       string to search
- *      s2       string to search for in s1
- *      s2_len   length in characters of s2
- *      skip     number of finds to ignore before returning anything
- *
- *  RETURNS: pointer to last occurrence of string s2 in string s1
- *           NULL if not found or error
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
+size_t my_strrspn(const char *s, const char *accept)
 {
-    char *p;
-    int comp;
-
-    if (!s2 || *s2 == '\0') {
-        return (char *) s1;
+    if (!s || *s == '\0') {
+        return 0;
     }
-    if (!s1 || *s1 == '\0') {
-        return NULL;
-    }
-    if (skip < 0) {
-        skip = 0;
+    if (!accept || *accept == '\0') {
+        return 0;
     }
 
-    p = strrchr(s1, s2[0]);
-    if (!p) {
-        return NULL;
-    }
-
-    while (p >= s1) {
-        comp = strncmp(p, s2, s2_len);
-        if (comp == 0) {
-            if (skip--) {
-                --p;
-            } else {
-                return p;
-            }
-        }
-        else {
-            --p;
+    for (int i = strlen(s) - 1; i >= 0; i--) {
+        size_t idx = (size_t) i;
+        if (strchr(accept, s[idx]) == NULL) {
+            return strlen(s) - (idx + 1);
         }
     }
-
-    return NULL;
+    return strlen(s);
 }
 
 
 
-char *tabbify_indent(const size_t lineno, char *indentspc, const size_t indentspc_len)
-/*
- *  Checks if tab expansion mode is "keep", and if so, calculates a new
- *  indentation string based on the one given. The new string contains
- *  tabs in their original positions.
- *
- *    lineno         index of the input line we are referring to
- *    indentspc      previously calculated "space-only" indentation string
- *                   (may be NULL). This is only used when opt.tabexp != 'k',
- *                   in which case it will be used as the function result.
- *    indentspc_len  desired result length, measured in spaces only
- *
- *  RETURNS:  if successful and opt.tabexp == 'k': new string
- *            on error (invalid input or out of memory): NULL
- *
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- */
+uint32_t *tabbify_indent(const size_t lineno, uint32_t *indentspc, const size_t indentspc_len)
 {
     size_t i;
-    char *result;
+    uint32_t *result;
     size_t result_len;
 
     if (opt.tabexp != 'k') {
@@ -458,29 +373,27 @@ char *tabbify_indent(const size_t lineno, char *indentspc, const size_t indentsp
         return NULL;
     }
     if (indentspc_len == 0) {
-        return (char *) strdup("");
+        return new_empty_string32();
     }
 
-    result = (char *) malloc(indentspc_len + 1);
+    result = (uint32_t *) malloc((indentspc_len + 1) * sizeof(uint32_t));
     if (result == NULL) {
         perror(PROJECT);
         return NULL;
     }
-    memset(result, (int) ' ', indentspc_len);
-    result[indentspc_len] = '\0';
+    u32_set(result, char_space, indentspc_len);
+    set_char_at(result, indentspc_len, char_nul);
     result_len = indentspc_len;
 
-    for (i = 0; i < input.lines[lineno].tabpos_len
-            && input.lines[lineno].tabpos[i] < indentspc_len; ++i)  /**/
-    {
+    for (i = 0; i < input.lines[lineno].tabpos_len && input.lines[lineno].tabpos[i] < indentspc_len; ++i) {
         size_t tpos = input.lines[lineno].tabpos[i];
-        size_t nspc = opt.tabstop - (tpos % opt.tabstop);   /* no of spcs covered by tab */
+        size_t nspc = opt.tabstop - (tpos % opt.tabstop); /* no of spcs covered by tab */
         if (tpos + nspc > input.indent) {
             break;
         }
-        result[tpos] = '\t';
+        set_char_at(result, tpos, char_tab);
         result_len -= nspc - 1;
-        result[result_len] = '\0';
+        set_char_at(result, result_len, char_nul);
     }
 
     return result;
@@ -498,42 +411,62 @@ char *nspaces(const size_t n)
 
 
 /**
- *  Debugging Code: Display contents of input structure
+ * Debugging Code: Display contents of input structure
+ * @param heading a heading to show for identification of the printed lines
  */
 void print_input_lines(const char *heading)
 {
-    fprintf(stderr, "Input Lines%s:\n", heading != NULL ? heading : "");
-    fprintf(stderr, "     [num_chars] \"real text\" [num_cols] \"ascii_text\"\n");
-    for (size_t i = 0; i < input.num_lines; ++i) {
-        fprintf(stderr, "%4d [%02d] \"%s\"  [%02d] \"%s\"", (int) i,
-                (int) input.lines[i].num_chars, u32_strconv_to_output(input.lines[i].mbtext),
-                (int) input.lines[i].len, input.lines[i].text);
-        fprintf(stderr, "\tTabs: [");
-        if (input.lines[i].tabpos != NULL) {
-            for (size_t j = 0; j < input.lines[i].tabpos_len; ++j) {
-                fprintf(stderr, "%d", (int) input.lines[i].tabpos[j]);
-                if (j < input.lines[i].tabpos_len - 1) {
-                    fprintf(stderr, ", ");
+    if (is_debug_logging(MAIN)) {
+        log_debug(__FILE__, MAIN, "%d Input Lines%s:\n", (int) input.num_lines, heading != NULL ? heading : "");
+        log_debug(__FILE__, MAIN, "     [num_chars] \"real text\" [num_cols] \"ascii_text\"\n");
+        for (size_t i = 0; i < input.num_lines; ++i) {
+            char *outtext = bxs_to_output(input.lines[i].text);
+            log_debug(__FILE__, MAIN, "%4d [%02d] \"%s\"  [%02d] \"%s\"", (int) i,
+                    (int) input.lines[i].text->num_chars, outtext,
+                    (int) input.lines[i].text->num_columns, input.lines[i].text->ascii);
+            BFREE(outtext);
+            log_debug_cont(MAIN, "\tTabs: [");
+            if (input.lines[i].tabpos != NULL) {
+                for (size_t j = 0; j < input.lines[i].tabpos_len; ++j) {
+                    log_debug_cont(MAIN, "%d", (int) input.lines[i].tabpos[j]);
+                    if (j < input.lines[i].tabpos_len - 1) {
+                        log_debug_cont(MAIN, ", ");
+                    }
                 }
             }
-        }
-        fprintf(stderr, "] (%d)", (int) input.lines[i].tabpos_len);
-        fprintf(stderr, "\tinvisible=%d\n", (int) input.lines[i].invis);
+            log_debug_cont(MAIN, "] (%d)", (int) input.lines[i].tabpos_len);
+            log_debug_cont(MAIN, "\tinvisible=%d\n", (int) input.lines[i].text->num_chars_invisible);
 
-        fprintf(stderr, "          posmap=");
-        if (input.lines[i].posmap != NULL) {
-            fprintf(stderr, "[");
-            for (size_t j = 0; j < input.lines[i].len; j++) {
-                fprintf(stderr, "%d%s", (int) input.lines[i].posmap[j], j == (input.lines[i].len - 1) ? "" : ", ");
+            log_debug(__FILE__, MAIN, "    visible_char=");
+            if (input.lines[i].text->visible_char != NULL) {
+                log_debug_cont(MAIN, "[");
+                for (size_t j = 0; j < input.lines[i].text->num_chars_visible; j++) {
+                    log_debug_cont(MAIN, "%d%s", (int) input.lines[i].text->visible_char[j],
+                        j == (input.lines[i].text->num_chars_visible - 1) ? "" : ", ");
+                }
+                log_debug_cont(MAIN, "]\n");
+            }
+            else {
+                log_debug_cont(MAIN, "null\n");
+            }
+
+            log_debug(__FILE__, MAIN, "      first_char=");
+            if (input.lines[i].text->first_char != NULL) {
+                log_debug_cont(MAIN, "[");
+                for (size_t j = 0; j < input.lines[i].text->num_chars_visible; j++) {
+                    log_debug_cont(MAIN, "%d%s", (int) input.lines[i].text->first_char[j],
+                        j == (input.lines[i].text->num_chars_visible - 1) ? "" : ", ");
+                }
+                log_debug_cont(MAIN, "]\n");
+            }
+            else {
+                log_debug_cont(MAIN, "null\n");
             }
-            fprintf(stderr, "]\n");
-        } else {
-            fprintf(stderr, "null\n");
         }
+        log_debug(__FILE__, MAIN, " Longest line: %d columns\n", (int) input.maxline);
+        log_debug(__FILE__, MAIN, "  Indentation: %2d spaces\n", (int) input.indent);
+        log_debug(__FILE__, MAIN, "Final newline: %s\n", input.final_newline ? "yes" : "no");
     }
-    fprintf(stderr, " Longest line: %d columns\n", (int) input.maxline);
-    fprintf(stderr, "  Indentation: %2d spaces\n", (int) input.indent);
-    fprintf(stderr, "Final newline: %s\n", input.final_newline ? "yes" : "no");
 }
 
 
@@ -551,10 +484,10 @@ void print_input_lines(const char *heading)
  * @param <posmap> pointer to the position map, which maps each position in <ascii> to a position in <s>
  * @returns the number of invisible characters in <s>
  */
-static size_t count_invisible_chars(const uint32_t *s, size_t *num_esc, char **ascii, size_t **posmap)
+size_t count_invisible_chars(const uint32_t *s, size_t *num_esc, char **ascii, size_t **posmap)
 {
-    size_t invis = 0;  /* counts invisible column positions */
-    *num_esc = 0;      /* counts the number of escape sequences found */
+    size_t invis = 0; /* counts invisible column positions */
+    *num_esc = 0;     /* counts the number of escape sequences found */
 
     if (is_empty(s)) {
         (*ascii) = (char *) strdup("");
@@ -565,8 +498,8 @@ static size_t count_invisible_chars(const uint32_t *s, size_t *num_esc, char **a
     size_t buflen = (size_t) u32_strwidth(s, encoding) + 1;
     size_t map_size = BMAX((size_t) 5, buflen);
     size_t map_idx = 0;
-    size_t *map = (size_t *) calloc(map_size, sizeof(size_t));  /* might not be enough if many double-wide chars */
-    (*ascii) = (char *) calloc(buflen, sizeof(char));     /* maybe a little too much, but certainly enough */
+    size_t *map = (size_t *) calloc(map_size, sizeof(size_t)); /* might not be enough if many double-wide chars */
+    (*ascii) = (char *) calloc(buflen, sizeof(char));          /* maybe a little too much, but certainly enough */
     char *p = *ascii;
 
     size_t mb_idx = 0;
@@ -611,40 +544,54 @@ static size_t count_invisible_chars(const uint32_t *s, size_t *num_esc, char **a
 
 
 
-void analyze_line_ascii(input_t *input_ptr, line_t *line)
+int is_csi_reset(const uint32_t *csi)
 {
-    size_t num_esc = 0;
-    char *ascii;
-    size_t *map;
-    size_t invis = count_invisible_chars(line->mbtext, &num_esc, &ascii, &(map));
-    line->invis = invis;
-    /* u32_strwidth() does not count control characters, i.e. ESC characters, for which we must correct */
-    line->len = u32_strwidth(line->mbtext, encoding) - invis + num_esc;
-    line->num_chars = u32_strlen(line->mbtext);
-    BFREE(line->text);
-    line->text = ascii;
-    BFREE(line->posmap);
-    line->posmap = map;
-
-    if (line->len > input_ptr->maxline) {
-        input_ptr->maxline = line->len;
+    ucs4_t puc = '\0';
+    const uint32_t *rest = csi;
+    size_t csi_pos = 0;
+    while ((rest = u32_next(&puc, rest))) {
+        switch(csi_pos) {
+            case 0:
+                if (puc != char_esc) {
+                    return 0;
+                }
+                break;
+            case 1:
+                if (puc != '[' && puc != '(') {
+                    return 0;
+                }
+                break;
+            case 2:
+                if (puc != '0') {
+                    if (puc >= 0x40 && puc <= 0x7e) {
+                        return 1;
+                    }
+                    else if ((puc == '1' && *rest == '0')
+                          || (puc == '3' && *rest == '9')
+                          || (puc == '4' && *rest == '9')
+                          || (puc == '5' && *rest == '9')
+                          || (puc == '7' && *rest == '5')) {
+                        rest = u32_next(&puc, rest);
+                        break;
+                    }
+                    return 0;
+                }
+                break;
+            default:
+                return (puc >= 0x40 && puc <= 0x7e) ? 1 : 0;
+        }
+        csi_pos++;
     }
+    return 0;
 }
 
 
 
-int array_contains(char **array, const size_t array_len, const char *s)
+void analyze_line_ascii(input_t *input_ptr, line_t *line)
 {
-    int result = 0;
-    if (array != NULL && array_len > 0) {
-        for (size_t i = 0; i < array_len; ++i) {
-            if (strcmp(array[i], s) == 0) {
-                result = 1;
-                break;
-            }
-        }
+    if (line->text->num_columns > input_ptr->maxline) {
+        input_ptr->maxline = line->text->num_columns;
     }
-    return result;
 }
 
 
@@ -665,6 +612,22 @@ int array_contains0(char **array, const char *s)
 
 
 
+int array_contains_bxs(bxstr_t **array, const size_t array_len, bxstr_t *s)
+{
+    int result = 0;
+    if (array != NULL && array_len > 0) {
+        for (size_t i = 0; i < array_len; ++i) {
+            if (bxs_strcmp(array[i], s) == 0) {
+                result = 1;
+                break;
+            }
+        }
+    }
+    return result;
+}
+
+
+
 size_t array_count0(char **array)
 {
     size_t num_elems = 0;
@@ -696,17 +659,19 @@ char *trimdup(char *s, char *e)
 
 int tag_is_valid(char *tag)
 {
-    if (tag == NULL) {
+    pcre2_code *pattern = get_pattern_ascii_id(1);
+    return regex_match(pattern, tag);
+}
+
+
+
+int is_ascii_id(bxstr_t *s, int strict)
+{
+    if (s == NULL || s->num_chars == 0) {
         return 0;
     }
-
-    const size_t len = strlen(tag);
-    return len > 0
-        && strspn(tag, "abcdefghijklmnopqrstuvwxyz-0123456789") == len
-        && strchr("abcdefghijklmnopqrstuvwxyz", tag[0]) != NULL
-        && tag[len - 1] != '-'
-        && strstr(tag, "--") == NULL
-        && strcmp(tag, "none") != 0;
+    pcre2_code *pattern = get_pattern_ascii_id(strict);
+    return u32_regex_match(pattern, s->memory);
 }
 
 
@@ -733,7 +698,7 @@ char *bx_strndup(const char *s, size_t n)
 
 
 
-void bx_fprintf(FILE *stream, const char *format, ...)
+void bx_fprintf_original(FILE *stream, const char *format, ...)
 {
     va_list va;
     va_start(va, format);
@@ -742,4 +707,29 @@ void bx_fprintf(FILE *stream, const char *format, ...)
 }
 
 
-/*EOF*/                                                  /* vim: set sw=4: */
+void set_bx_fprintf(bx_fprintf_t bx_fprintf_function) {
+    bx_fprintf = bx_fprintf_function;
+}
+
+
+FILE *bx_fopens(bxstr_t *pathname, char *mode)
+{
+    return bx_fopen(to_utf8(pathname->memory), mode);
+}
+
+
+
+FILE *bx_fopen(char *pathname, char *mode)
+{
+    /*
+     * On Linux/UNIX and OS X (Mac), one can access files with non-ASCII file names by passing them to fopen() as UTF-8.
+     * On Windows, a different function must be called. (Info: https://stackoverflow.com/a/35065142/1005481)
+     * On newer Windows, we're good:
+     * https://learn.microsoft.com/en-us/cpp/c-runtime-library/reference/setlocale-wsetlocale#utf-8-support
+     */
+    FILE *f = fopen(pathname, mode);
+    return f;
+}
+
+
+/* vim: set sw=4: */
