@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,10 +18,12 @@
  */
 
 #include "config.h"
+
 #include <stdarg.h>
 #include <stddef.h>
 #include <setjmp.h>
 #include <cmocka.h>
+
 #include "boxes.h"
 #include "cmdline.h"
 #include "tools.h"
@@ -58,7 +60,7 @@ static opt_t *act(const int num_args, ...)
 
 void test_indentmode_none(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-i", "none");
 
@@ -69,7 +71,7 @@ void test_indentmode_none(void **state)
 
 void test_indentmode_invalid_long(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-i", "INVALID");
 
@@ -81,7 +83,7 @@ void test_indentmode_invalid_long(void **state)
 
 void test_indentmode_invalid_short(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-i", "X");
 
@@ -93,7 +95,7 @@ void test_indentmode_invalid_short(void **state)
 
 void test_indentmode_box(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-i", "BO");
 
@@ -104,7 +106,7 @@ void test_indentmode_box(void **state)
 
 void test_indentmode_text(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-i", "t");
 
@@ -115,7 +117,7 @@ void test_indentmode_text(void **state)
 
 void test_killblank_true(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-k", "true");
 
@@ -126,7 +128,7 @@ void test_killblank_true(void **state)
 
 void test_killblank_false(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-k", "false");
 
@@ -137,7 +139,7 @@ void test_killblank_false(void **state)
 
 void test_killblank_invalid(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-k", "INVALID");
 
@@ -149,7 +151,7 @@ void test_killblank_invalid(void **state)
 
 void test_killblank_multiple(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(4, "-k", "true", "-k", "false");    // first one wins
 
@@ -158,9 +160,25 @@ void test_killblank_multiple(void **state)
 }
 
 
+void test_killblank_long(void **state)
+{
+    UNUSED(state);
+
+    opt_t *actual = act(2, "--kill-blank", "--remove");
+
+    assert_non_null(actual);
+    assert_int_equal(1, actual->killblank);
+
+    actual = act(2, "--no-kill-blank", "--remove");
+
+    assert_non_null(actual);
+    assert_int_equal(0, actual->killblank);
+}
+
+
 void test_padding_top_bottom(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "t2b10");
 
@@ -172,7 +190,7 @@ void test_padding_top_bottom(void **state)
 
 void test_padding_invalid(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "INVALID");
 
@@ -184,7 +202,7 @@ void test_padding_invalid(void **state)
 
 void test_padding_negative(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "a-1");
 
@@ -196,7 +214,7 @@ void test_padding_negative(void **state)
 
 void test_padding_notset(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "");
 
@@ -208,7 +226,7 @@ void test_padding_notset(void **state)
 
 void test_padding_invalid_value(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "l2rX");
 
@@ -220,7 +238,7 @@ void test_padding_invalid_value(void **state)
 
 void test_padding_novalue(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-p", "a");
 
@@ -232,7 +250,7 @@ void test_padding_novalue(void **state)
 
 void test_tabstops_zero(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "0");
 
@@ -244,7 +262,7 @@ void test_tabstops_zero(void **state)
 
 void test_tabstops_500(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "500");
 
@@ -256,7 +274,7 @@ void test_tabstops_500(void **state)
 
 void test_tabstops_4X(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "4X");
 
@@ -268,7 +286,7 @@ void test_tabstops_4X(void **state)
 
 void test_tabstops_4e(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "4e");
 
@@ -280,7 +298,7 @@ void test_tabstops_4e(void **state)
 
 void test_tabstops_4ex(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "4ex");
 
@@ -292,7 +310,7 @@ void test_tabstops_4ex(void **state)
 
 void test_tabstops_7(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-t", "7");
 
@@ -304,7 +322,7 @@ void test_tabstops_7(void **state)
 
 void test_alignment_invalid_hX(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-a", "hX");
 
@@ -316,7 +334,7 @@ void test_alignment_invalid_hX(void **state)
 
 void test_alignment_invalid_vX(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-a", "vX");
 
@@ -328,7 +346,7 @@ void test_alignment_invalid_vX(void **state)
 
 void test_alignment_invalid_jX(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-a", "jX");
 
@@ -340,7 +358,7 @@ void test_alignment_invalid_jX(void **state)
 
 void test_alignment_notset(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-a", "");
 
@@ -352,7 +370,7 @@ void test_alignment_notset(void **state)
 
 void test_alignment_incomplete(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-a", "v");
 
@@ -364,7 +382,7 @@ void test_alignment_incomplete(void **state)
 
 void test_inputfiles_illegal_third_file(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(3, "file1", "file2", "file3_ILLEGAL");
 
@@ -378,7 +396,7 @@ void test_inputfiles_illegal_third_file(void **state)
 
 void test_inputfiles_stdin_stdout(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-", "-");
 
@@ -390,7 +408,7 @@ void test_inputfiles_stdin_stdout(void **state)
 
 void test_inputfiles_stdin(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(1, "-");
 
@@ -402,7 +420,7 @@ void test_inputfiles_stdin(void **state)
 
 void test_inputfiles_input_nonexistent(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(1, "NON-EXISTENT");
 
@@ -414,9 +432,9 @@ void test_inputfiles_input_nonexistent(void **state)
 
 void test_inputfiles_actual_success(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
-    opt_t *actual = act(2, "../utest/dummy_in.txt", "dummy_out.txt");
+    opt_t *actual = act(2, "../utest/dummy_in.txt", "../out/dummy_out.txt");
 
     assert_non_null(actual);
     assert_non_null(actual->infile);
@@ -426,7 +444,7 @@ void test_inputfiles_actual_success(void **state)
 
 void test_command_line_design_empty(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(2, "-c", "");
 
@@ -438,7 +456,7 @@ void test_command_line_design_empty(void **state)
 
 void test_help(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(1, "-h");
 
@@ -450,7 +468,7 @@ void test_help(void **state)
 
 void test_version_requested(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     opt_t *actual = act(1, "-v");
 
@@ -460,4 +478,4 @@ void test_version_requested(void **state)
 }
 
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
