@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -31,6 +31,7 @@ void test_killblank_true(void **state);
 void test_killblank_false(void **state);
 void test_killblank_invalid(void **state);
 void test_killblank_multiple(void **state);
+void test_killblank_long(void **state);
 
 void test_padding_top_bottom(void **state);
 void test_padding_invalid(void **state);
@@ -66,4 +67,4 @@ void test_version_requested(void **state);
 
 #endif
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
