@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,14 +18,19 @@
  */
 
 #include "config.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
 #include <uniconv.h>
+
 #include "boxes.h"
 #include "unicode.h"
 #include "tools.h"
+#include "global_mock.h"
+
+
 
 design_t *designs = NULL;
 
@@ -35,6 +40,8 @@ opt_t opt;
 
 input_t input;
 
+int color_output_enabled = 1;
+
 char **collect_out = NULL;
 int collect_out_size = 0;
 
@@ -61,7 +68,7 @@ void collect_reset()
 
 /**
  * Mock of the `bx_fprintf()` function which records its output instead of printing it. Assumes that no output string
- * will be longer than 512 characters.
+ * of our test cases will be longer than 512 characters.
  * @param stream `stdout` or `stderr`
  * @param format the format string, followed by the arguments
  */
@@ -94,6 +101,7 @@ void setup_mocks()
     setlocale(LC_ALL, "");
     encoding = check_encoding("UTF-8", locale_charset());
     collect_reset();
+    set_bx_fprintf(__wrap_bx_fprintf);
 }
 
 
@@ -103,4 +111,4 @@ void teardown()
 }
 
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
