@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -18,20 +18,24 @@
  */
 
 #include "config.h"
+
 #include <stdarg.h>
 #include <stddef.h>
 #include <setjmp.h>
 #include <cmocka.h>
 #include <uniconv.h>
 #include <string.h>
+
+#include "boxes.h"
 #include "global_mock.h"
 #include "regulex.h"
+#include "regulex_test.h"
 
 
 
 void test_compile_pattern_empty(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     assert_null(compile_pattern(NULL));
     assert_non_null(compile_pattern(""));
@@ -41,7 +45,7 @@ void test_compile_pattern_empty(void **state)
 
 void test_compile_pattern_error(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     assert_null(compile_pattern("incomplete[x"));
     assert_int_equal(1, collect_err_size);
@@ -59,7 +63,7 @@ void test_compile_pattern_error(void **state)
 
 void test_regex_replace_invalid_utf(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     const char *input = "input";
     assert_null(regex_replace(compile_pattern("search"), NULL, /* NULL is an invalid replacement string*/
@@ -72,7 +76,7 @@ void test_regex_replace_invalid_utf(void **state)
 
 void test_regex_replace_buffer_resize(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     const char *input = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
     uint32_t *actual = regex_replace(compile_pattern("x"), "long_replacement_string_",
@@ -95,7 +99,7 @@ void test_regex_replace_buffer_resize(void **state)
 
 void test_regex_replace_error(void **state)
 {
-    (void) state;  /* unused */
+    UNUSED(state);
 
     const char *input = "xxx";
     uint32_t *actual = regex_replace(compile_pattern("x"), "INVALID $2",
@@ -106,4 +110,4 @@ void test_regex_replace_error(void **state)
 }
 
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
