@@ -1,6 +1,6 @@
 /*
  * boxes - Command line filter to draw/remove ASCII boxes around text
- * Copyright (c) 1999-2023 Thomas Jensen and the boxes contributors
+ * Copyright (c) 1999-2024 Thomas Jensen and the boxes contributors
  *
  * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
  * License, version 3, as published by the Free Software Foundation.
@@ -14,7 +14,7 @@
  */
 
 /*
- * Unit tests of the 'tools' module
+ * Unit tests of the 'tools' module.
  */
 
 #ifndef TOOLS_TEST_H
@@ -27,7 +27,19 @@ void test_strisyes_false(void **state);
 void test_strisno_true(void **state);
 void test_strisno_false(void **state);
 
+void test_my_strrspn_edge(void **state);
+void test_my_strrspn(void **state);
+
+void test_is_csi_reset(void **state);
+
+void test_is_ascii_id_valid(void **state);
+void test_is_ascii_id_invalid(void **state);
+void test_is_ascii_id_strict_valid(void **state);
+void test_is_ascii_id_strict_invalid(void **state);
+
+void test_repeat(void **state);
+
 
 #endif
 
-/*EOF*/                                          /* vim: set cindent sw=4: */
+/* vim: set cindent sw=4: */
